; 6502 Values and Macros
; License: GPL-2, see LICENSE
; Date Created: 2022-06-17
; (c) 2022 Mark Ormston <memso@memso.net>

.IFNDEF IncludeFile_6502
IncludeFile_6502 = 1

; #################################################################################################
; #################################################################################################
; #################################################################################################
; Addresses

; Address of the zero page.
ZeroPage            = $00

; Address of the stack.
CPUStack            = $0100

; NMI vector.
NMIVector           = $FFFA

; Reset vector.
ResetVector         = $FFFC

; IRQ vector.
IRQVector           = $FFFE



; #################################################################################################
; #################################################################################################
; #################################################################################################
; Constants

; A dummy address for self-modifying code.
DUMMYADDRESS = $FFFE

; The size of vectors in vector tables. This is 3 bytes because they're in JMP $abcd format.
VECTORSIZE = 3



; #################################################################################################
; #################################################################################################
; #################################################################################################
; Enums

; *************************************************************************************************
; Opcodes
.ENUM OPCODE
	; $00 - $0F =====================================================================================
	BRK_Imp           = $00
	ORA_ZPIntX        = $01
	;HLT_Imp_02        = $02
	;ASO_ZPIndX        = $03
	;SKB_ZP            = $04
	ORA_ZP            = $05
	ASL_ZP            = $06
	;ASO_ZP            = $07
	PHP_Imp           = $08
	ORA_Imm           = $09
	ASL_A             = $0A
	;ANC_Imm           = $0B
	;SKW_Abs           = $0C
	ORA_Abs           = $0D
	ASL_Abs           = $0E
	;ASO_Abs           = $0F


	; $10 - $1F =====================================================================================
	BPL_Near          = $10
	ORA_ZPIndY        = $11
	;HLT_Imp_12        = $12
	;ASO_ZPIndY        = $13
	;SKB_ZPX           = $14
	ORA_ZPX           = $15
	ASL_ZPX           = $16
	;ASO_ZPX           = $17
	CLC_Imp           = $18
	ORA_AbsY          = $19
	;NOP_Imp_1A        = $1A
	;ASO_AbsY          = $1B
	;SKW_AbsX          = $1C
	ORA_AbsX          = $1D
	ASL_AbsX          = $1E
	;ASO_AbsX         = $1F


	; $20 - $2F =====================================================================================
	JSR_Abs           = $20
	AND_ZPIndX        = $21
	;HLT_Imp_22        = $22
	;RLA_ZPIndX        = $23
	BIT_ZP            = $24
	AND_ZP            = $25
	ROL_ZP            = $26
	;RLA_ZP            = $27
	PLP_Imp           = $28
	AND_Imm           = $29
	ROL_A             = $2A
	;ANC_Imm           = $2B
	BIT_Abs           = $2C
	AND_Abs           = $2D
	ROL_Abs           = $2E
	;RLA_Abs           = $2F


	; $30 - $3F =====================================================================================
	BMI_Near          = $30
	AND_ZPIndY        = $31
	AND_ZPX           = $35
	ROL_ZPX           = $36
	SEC_Imp           = $38
	AND_AbsY          = $39
	AND_AbsX          = $3D
	ROL_AbsX          = $3E


	; $40 - $4F =====================================================================================
	RTI_Imp           = $40
	EOR_ZPIndX        = $41
	EOR_ZP            = $45
	LSR_ZP            = $46
	PHA_Imp           = $48
	EOR_Imm           = $49
	LSR_A             = $4A
	JMP_Abs           = $4C
	EOR_Abs           = $4D
	LSR_Abs           = $4E


	; $50 - $5F =====================================================================================
	BVC_Near          = $50
	EOR_ZPIndY        = $51
	EOR_ZPX           = $55
	LSR_ZPX           = $56
	CLI_Imp           = $58
	EOR_AbsY          = $59
	EOR_AbsX          = $5D
	LSR_AbsX          = $5E


	; $60 - $6F =====================================================================================
	RTS_Imp           = $60
	ADC_ZPIndX        = $61
	;HLT_Imp           = $62
	;RRA_ZPX           = $63
	;SKB_ZP            = $64
	ADC_ZP            = $65
	ROR_ZP            = $66
	;RRA_ZP            = $67
	PLA_Imp           = $68
	ADC_Imm           = $69
	ROR_A             = $6A
	;ARR_Imm           = $6B
	JMP_AbsInd        = $6C
	ADC_Abs           = $6D
	ROR_Abs           = $6E
	;RRA_Abs           = $6F


	; $70 - $7F =====================================================================================
	BVS_Near          = $70
	ADC_ZPIndY        = $71
	ADC_ZPX           = $75
	ROR_ZPX           = $76
	SEI_Imp           = $78
	ADC_AbsY          = $79
	ADC_AbsX          = $7D
	ROR_AbsX          = $7E


	; $80 - $8F =====================================================================================
	;SKB_Imm           = $80
	STA_ZPIndX        = $81
	STY_ZP            = $84
	STA_ZP            = $85
	STX_ZP            = $86
	DEY_Imp           = $88
	TXA_Imp           = $8A
	STY_Abs           = $8C
	STA_Abs           = $8D
	STX_Abs           = $8E


	; $90 - $9F =====================================================================================
	BCC_Near          = $90
	STA_ZPIndY        = $91
	STY_ZPX           = $94
	STA_ZPX           = $95
	STX_ZPY           = $96
	TYA_Imp           = $98
	STA_AbsY          = $99
	TXS_Imp           = $9A
	STA_AbsX          = $9D


	; $A0 - $AF =====================================================================================
	LDY_Imm           = $A0
	LDA_ZPIndX        = $A1
	LDX_Imm           = $A2
	LDY_ZP            = $A4
	LDA_ZP            = $A5
	LDX_ZP            = $A6
	TAY_Imp           = $A8
	LDA_Imm           = $A9
	TAX_Imp           = $AA
	LDY_Abs           = $AC
	LDA_Abs           = $AD
	LDX_Abs           = $AE


	; $B0 - $BF =====================================================================================
	BCS_Near          = $B0
	LDA_ZPIndY        = $B1
	LDY_ZPX           = $B4
	LDA_ZPX           = $B5
	LDX_ZPY           = $B6
	CLV_Imp           = $B8
	LDA_AbsY          = $B9
	TSX_Imp           = $BA
	LDY_AbsX          = $BC
	LDA_AbsX          = $BD
	LDX_AbsY          = $BE


	; $C0 - $CF =====================================================================================
	CPY_Imm           = $C0
	CMP_ZPIndX        = $C1
	CPY_ZP            = $C4
	CMP_ZP            = $C5
	DEC_ZP            = $C6
	INY_Imp           = $C8
	CMP_Imm           = $C9
	DEX_Imp           = $CA
	CPY_Abs           = $CC
	CMP_Abs           = $CD
	DEC_Abs           = $CE


	; $D0 - $DF =====================================================================================
	BNE_Near          = $D0
	CMP_ZPIndY        = $D1
	CMP_ZPX           = $D5
	DEC_ZPX           = $D6
	CLD_Imp           = $D8
	CMP_AbsY          = $D9
	CMP_AbsX          = $DD
	DEC_AbsX          = $DE


	; $E0 - $EF =====================================================================================
	CPX_Imm           = $E0
	SBC_ZPIndX        = $E1
	CPX_ZP            = $E4
	SBC_ZP            = $E5
	INC_ZP            = $E6
	INX_Imp           = $E8
	SBC_Imm           = $E9
	NOP_Imp           = $EA
	CPX_Abs           = $EC
	SBC_Abs           = $ED
	INC_Abs           = $EE


	; $F0 - $FF =====================================================================================
	BEQ_Near          = $F0
	SBC_ZPIndY        = $F1
	SBC_ZPX           = $F5
	INC_ZPX           = $F6
	SED_Imp           = $F8
	SBC_AbsY          = $F9
	SBC_AbsX          = $FD
	INC_AbsX          = $FE
.ENDENUM



; *************************************************************************************************
; Common Values

; =================================================================================================
; Bits
; These are included to make it clear when specifically choosing an immediate based on a bit value
; vs anything else.
.ENUM BITVALUE
	; Standard bit value list
	Bit0 = $01
	Bit1 = $02
	Bit2 = $04
	Bit3 = $08
	Bit4 = $10
	Bit5 = $20
	Bit6 = $40
	Bit7 = $80

	; Not values
	Not0 = $FE
	Not1 = $FD
	Not2 = $FB
	Not3 = $F7
	Not4 = $EF
	Not5 = $DF
	Not6 = $BF
	Not7 = $7F
.ENDENUM



; #################################################################################################
; #################################################################################################
; #################################################################################################
; Code Macros

; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Copying Values

; *@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*
; *@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*
; Copy a value directly

; *************************************************************************************************
; Copies an 8-bit value from one address to another.
; Incoming:
;   destination     Address, The destination.
;   source          Address, The source.
; Outgoing:
;   A is source.
;         ABS = ABS  ABS = ZP  ZP = ABS  ZP = ZP
; Size:   6          5         5         4
; Cycles: 8          7         7         6
.MACRO CopyValue destination, source
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF
	.IF .MATCH(.LEFT(1, {source}), #)
		.ERROR "Source is an immediate."
	.ENDIF

  LDA source
  STA destination
.ENDMACRO



; *************************************************************************************************
; Copies an 8-bit value from one address to another by X.
; Incoming:
;   destination     Address, The destination.
;   source          Address, The source.
; Outgoing:
;   X is source.
;         ABS = ABS  ABS = ZP  ZP = ABS  ZP = ZP
; Size:   6          5         5         4
; Cycles: 8          7         7         6
.MACRO CopyValueByX destination, source
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF
	.IF .MATCH(.LEFT(1, {source}), #)
		.ERROR "Source is an immediate."
	.ENDIF

  LDX source
  STX destination
.ENDMACRO



; *************************************************************************************************
; Copies an 8-bit value from one address to another by Y.
; Incoming:
;   destination     Address, The destination.
;   source          Address, The source.
; Outgoing:
;   Y is source.
;         ABS = ABS  ABS = ZP  ZP = ABS  ZP = ZP
; Size:   6          5         5         4
; Cycles: 8          7         7         6
.MACRO CopyValueByY destination, source
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF
	.IF .MATCH(.LEFT(1, {source}), #)
		.ERROR "Source is an immediate."
	.ENDIF

  LDY source
  STY destination
.ENDMACRO



; *************************************************************************************************
; Copies a 16-bit value from one address to another.
; Incoming:
;   destination     Address, The destination.
;   source          Address, The source.
; Outgoing:
;   A is source+1.
;         ABS = ABS  ABS = ZP  ZP = ABS  ZP = ZP
; Size:   12         10        10        8
; Cycles: 16         14        14        12
.MACRO CopyValue16 destination, source
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF
	.IF .MATCH(.LEFT(1, {source}), #)
		.ERROR "Source is an immediate."
	.ENDIF

  LDA source
  STA destination
  LDA source+1
  STA destination+1
.ENDMACRO



; *************************************************************************************************
; Copies a 16-bit value from one address to another by X.
; Incoming:
;   destination     Address, The destination.
;   source          Address, The source.
; Outgoing:
;   X is source+1.
;         ABS = ABS  ABS = ZP  ZP = ABS  ZP = ZP
; Size:   12         10        10        8
; Cycles: 16         14        14        12
.MACRO CopyValue16ByX destination, source
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF
	.IF .MATCH(.LEFT(1, {source}), #)
		.ERROR "Source is an immediate."
	.ENDIF

  LDX source
  STX destination
  LDX source+1
  STX destination+1
.ENDMACRO



; *************************************************************************************************
; Copies a 16-bit value from one address to another by Y.
; Incoming:
;   destination     Address, The destination.
;   source          Address, The source.
; Outgoing:
;   Y is source+1.
;         ABS = ABS  ABS = ZP  ZP = ABS  ZP = ZP
; Size:   12         10        10        8
; Cycles: 16         14        14        12
.MACRO CopyValue16ByY destination, source
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF
	.IF .MATCH(.LEFT(1, {source}), #)
		.ERROR "Source is an immediate."
	.ENDIF

  LDY source
  STY destination
  LDY source+1
  STY destination+1
.ENDMACRO



; *************************************************************************************************
; Copies a 16-bit value from one address to another by X & Y (X High).
; Incoming:
;   destination     Address, The destination.
;   source          Address, The source.
; Outgoing:
;   X & Y (X High) are the source.
;         ABS = ABS  ABS = ZP  ZP = ABS  ZP = ZP
; Size:   12         10        10        8
; Cycles: 16         14        14        12
.MACRO CopyValue16ByXY destination, source
	LoadXY source
	StoreXY destination
.ENDMACRO



; *************************************************************************************************
; Copies a value to multiple other values.
; Incoming:
;   source          Address, The source.
;   d[x]            Addresses, The destinations.
; Outgoing:
;   A is source
; Size:   (3 if source is ABS, 2 if ZP) + (# of ZP addresses * 2) + (# of ABS addresses * 3)
; Cycles: (4 if source is ABS, 3 if ZP) + (# of ZP addresses * 3) + (# of ABS addresses * 4)
.MACRO CopyToMultiple source, d0, d1, d2, d3, d4, d5, d6, d7, d8, d9, d10, d11, d12, d13, d14, d15
	; Verifcation ===================================================================================
	.IF .MATCH(.LEFT(1, {source}), #)
		.ERROR "Source is an immediate."
	.ENDIF


	; Copy ==========================================================================================
	LDA source
	.IFNBLANK d0
		STA d0
	.ENDIF
	.IFNBLANK d1
		STA d1
	.ENDIF
	.IFNBLANK d2
		STA d2
	.ENDIF
	.IFNBLANK d3
		STA d3
	.ENDIF
	.IFNBLANK d4
		STA d4
	.ENDIF
	.IFNBLANK d5
		STA d5
	.ENDIF
	.IFNBLANK d6
		STA d6
	.ENDIF
	.IFNBLANK d7
		STA d7
	.ENDIF
	.IFNBLANK d8
		STA d8
	.ENDIF
	.IFNBLANK d9
		STA d9
	.ENDIF
	.IFNBLANK d10
		STA d10
	.ENDIF
	.IFNBLANK d11
		STA d11
	.ENDIF
	.IFNBLANK d12
		STA d12
	.ENDIF
	.IFNBLANK d13
		STA d13
	.ENDIF
	.IFNBLANK d14
		STA d14
	.ENDIF
	.IFNBLANK d15
		STA d15
	.ENDIF
.ENDMACRO



; *************************************************************************************************
; Copies a 16-bit value to multiple other 16-bit values.
; Incoming:
;   source          Address, The source.
;   d[x]            Addresses, The destinations.
; Outgoing:
;   A is the high byte of source
; Size:   (6 if source is ABS, 4 if ZP) + (# of ZP addresses * 4) + (# of ABS addresses * 6)
; Cycles: (8 if source is ABS, 6 if ZP) + (# of ZP addresses * 6) + (# of ABS addresses * 8)
.MACRO CopyToMultiple16 source, d0, d1, d2, d3, d4, d5, d6, d7, d8, d9, d10, d11, d12, d13, d14, d15
	; Verifcation ===================================================================================
	.IF .MATCH(.LEFT(1, {source}), #)
		.ERROR "Source is an immediate."
	.ENDIF


	; Copy low byte =================================================================================
	LDA source
	.IFNBLANK d0
		STA d0
	.ENDIF
	.IFNBLANK d1
		STA d1
	.ENDIF
	.IFNBLANK d2
		STA d2
	.ENDIF
	.IFNBLANK d3
		STA d3
	.ENDIF
	.IFNBLANK d4
		STA d4
	.ENDIF
	.IFNBLANK d5
		STA d5
	.ENDIF
	.IFNBLANK d6
		STA d6
	.ENDIF
	.IFNBLANK d7
		STA d7
	.ENDIF
	.IFNBLANK d8
		STA d8
	.ENDIF
	.IFNBLANK d9
		STA d9
	.ENDIF
	.IFNBLANK d10
		STA d10
	.ENDIF
	.IFNBLANK d11
		STA d11
	.ENDIF
	.IFNBLANK d12
		STA d12
	.ENDIF
	.IFNBLANK d13
		STA d13
	.ENDIF
	.IFNBLANK d14
		STA d14
	.ENDIF
	.IFNBLANK d15
		STA d15
	.ENDIF


	; Copy high byte ================================================================================
	LDA source+1
	.IFNBLANK d0
		STA d0+1
	.ENDIF
	.IFNBLANK d1
		STA d1+1
	.ENDIF
	.IFNBLANK d2
		STA d2+1
	.ENDIF
	.IFNBLANK d3
		STA d3+1
	.ENDIF
	.IFNBLANK d4
		STA d4+1
	.ENDIF
	.IFNBLANK d5
		STA d5+1
	.ENDIF
	.IFNBLANK d6
		STA d6+1
	.ENDIF
	.IFNBLANK d7
		STA d7+1
	.ENDIF
	.IFNBLANK d8
		STA d8+1
	.ENDIF
	.IFNBLANK d9
		STA d9+1
	.ENDIF
	.IFNBLANK d10
		STA d10+1
	.ENDIF
	.IFNBLANK d11
		STA d11+1
	.ENDIF
	.IFNBLANK d12
		STA d12+1
	.ENDIF
	.IFNBLANK d13
		STA d13+1
	.ENDIF
	.IFNBLANK d14
		STA d14+1
	.ENDIF
	.IFNBLANK d15
		STA d15+1
	.ENDIF
.ENDMACRO



; *@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*
; *@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*
; Copy To/From a list by X offset

; *************************************************************************************************
; Copies an 8-bit value from a list with an X offset.
; Incoming:
;   destination     Address, The destination.
;   sourcelist      Address, The source list.
; Incoming:
;   X is the offset in the list to use.
; Outgoing:
;   A is the value stored.
;         ABS = ABS,X  ZP = ABS,X  ABS = ZP,X  ZP = ZP,X
; Size:   6            5           5           4
; Cycles: 8 to 9       7 to 8      8           7
.MACRO CopyFromListByX destination, sourcelist
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF

  LDA sourcelist, X
  STA destination
.ENDMACRO



; *************************************************************************************************
; Copies an 8-bit value to a list with an X offset.
; Incoming:
;   destlist        Address, The destination list.
;   source          Address, The source.
; Incoming:
;   X is the offset in the list to use.
; Outgoing:
;   A is source.
;         ABS,X = ABS  ABS,X = ZP  ZP,X = ABS  ZP,X = ZP
; Size:   6            5           5           4
; Cycles: 9            8           8           7
.MACRO CopyToListByX destlist, source
	.IF .MATCH(.LEFT(1, {source}), #)
		.ERROR "Source is an immediate."
	.ENDIF

  LDA source
  STA destlist, X
.ENDMACRO



; *************************************************************************************************
; Copies a 16-bit value from a list with an X offset.
; Incoming:
;   destination     Address, The destination.
;   sourcelistlow   Address, The low address for the source list.
;   sourcelisthigh  Optional - Address, The high address for the source list.
;                   Default: sourcelistlow+1
; Incoming:
;   X is the offset in the list to use.
; Outgoing:
;   A is the high value.
;         ABS = ABS,X  ZP = ABS,X  ABS = ZP,X  ZP = ZP,X
; Size:   12           10          10          8
; Cycles: 16 to 18     14 to 16    16          14
.MACRO CopyFromListByX16 destination, sourcelistlow, sourcelisthigh
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF

  LDA sourcelistlow, X
  STA destination
  .IFBLANK sourcelisthigh
  	LDA sourcelistlow+1, X
  .ELSE
  	LDA sourcelisthigh, X
  .ENDIF
  STA destination+1
.ENDMACRO



; *************************************************************************************************
; Copies a 16-bit value to a list with an X offset. This *MUST* have 3 parameters.
; Incoming:
;   destlistlow     Address, The low address for the destination list.
;   destlisthigh    Address, The high address for the destination list.
;   source          Address, The source.
; Incoming:
;   X is the offset in the list to use.
; Outgoing:
;   A is lost.
;         ABS,X = ABS  ABS,X = ZP  ZP,X = ABS  ZP,X = ZP
; Size:   12           10          10          8
; Cycles: 20           18          18          16
.MACRO CopyToListByX16 destlistlow, destlisthigh, source
	.IF .PARAMCOUNT <> 3
		.ERROR "Macro CopyToListByY16 must have 3 parameters."
	.ENDIF
	.IF .MATCH(.LEFT(1, {source}), #)
		.ERROR "Source is an immediate."
	.ENDIF

  LDA source
  STA destlistlow, X
  LDA source+1
 	STA destlisthigh, X
.ENDMACRO



; *@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*
; *@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*
; Copy to/from a list by Y

; *************************************************************************************************
; Copies an 8-bit value from a list with a Y offset.
; NOTE: The source is always accessed absolutely with this function. ZP, Y is not supported.
; Incoming:
;   destination     Address, The destination.
;   sourcelist      Address, The source list.
; Incoming:
;   Y is the offset in the list to use.
; Outgoing:
;   A is the value stored.
;         ABS = ABS,Y  ZP = ABS,Y
; Size:   6            5
; Cycles: 8 to 9       7 to 8
.MACRO CopyFromListByY destination, sourcelist
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF

  LDA sourcelist, Y
  STA destination
.ENDMACRO



; *************************************************************************************************
; Copies an 8-bit value to a list with a Y offset.
; NOTE: The destination is always accessed absolutely with this function. ZP, Y is not supported.
; Incoming:
;   destlist        Address, The destination list.
;   source          Address, The source.
; Incoming:
;   Y is the offset in the list to use.
; Outgoing:
;   A is source.
;         ABS,Y = ABS  ABS,Y = ZP
; Size:   6            5
; Cycles: 9            8
.MACRO CopyToListByY destlist, source
	.IF .MATCH(.LEFT(1, {source}), #)
		.ERROR "Source is an immediate."
	.ENDIF

  LDA source
  STA destlist, Y
.ENDMACRO



; *************************************************************************************************
; Copies a 16-bit value from a list with a Y offset.
; NOTE: The source is always accessed absolutely with this function. ZP, Y is not supported.
; Incoming:
;   destination     Address, The destination.
;   sourcelistlow   Address, The low address for the source list.
;   sourcelisthigh  Optional - Address, The high address for the source list.
;                   Default: sourcelistlow+1
; Incoming:
;   Y is the offset in the list to use.
; Outgoing:
;   A is the high byte.
;         ABS = ABS,Y  ZP = ABS,Y
; Size:   12           10
; Cycles: 16 to 18     14 to 16
.MACRO CopyFromListByY16 destination, sourcelistlow, sourcelisthigh
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF

  LDA sourcelistlow, Y
  STA destination
  .IFBLANK sourcelisthigh
  	LDA sourcelistlow+1, Y
  .ELSE
  	LDA sourcelisthigh, Y
  .ENDIF
  STA destination+1
.ENDMACRO



; *************************************************************************************************
; Copies a 16-bit value to a list with a Y offset. This *MUST* have 3 parameters.
; NOTE: The destination is always accessed absolutely with this function. ZP, Y is not supported.
; Incoming:
;   destlistlow     Address, The low address for the destination list.
;   destlisthigh    Address, The high address for the destination list.
;   source          Address, The source.
; Incoming:
;   Y is the offset in the list to use.
; Outgoing:
;   A is source+1.
;         ABS,Y = ABS  ABS,Y = ZP
; Size:   12           10
; Cycles: 18           16
.MACRO CopyToListByY16 destlistlow, destlisthigh, source
	.IF .PARAMCOUNT <> 3
		.ERROR "Macro CopyToListByY16 must have 3 parameters."
	.ENDIF
	.IF .MATCH(.LEFT(1, {source}), #)
		.ERROR "Source is an immediate."
	.ENDIF

  LDA source
  STA destlistlow, Y
  LDA source+1
 	STA destlisthigh, Y
.ENDMACRO



; *@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*
; *@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*
; Copy to/from a list by zero page Y

; *************************************************************************************************
; Copies an 8-bit value from a ZP pointer list by Y.
; Incoming:
;   destination     Address, The destination.
;   sourcelist      ZP Address, The source ZPY list.
; Incoming:
;   Y is the offset in the list to use.
; Outgoing:
;   A is the value stored.
;         ABS      ZP
; Size:   5        4
; Cycles: 9 to 10  8 to 9
.MACRO CopyFromZPYList destination, sourcelist
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF

	LDA (sourcelist), Y
	STA destination
.ENDMACRO



; *************************************************************************************************
; Copies an 8-bit value to a ZP pointer list by Y.
; Incoming:
;   destlist        ZP Address, The destination ZPY list.
;   source          Address, The source.
; Incoming:
;   Y is the offset in the list to use.
; Outgoing:
;   A is the value stored.
;         ABS  ZP
; Size:   5    4
; Cycles: 10   9
.MACRO CopyToZPYList destlist, source
	.IF .MATCH(.LEFT(1, {source}), #)
		.ERROR "Source is an immediate."
	.ENDIF

	LDA source
	STA (destlist), Y
.ENDMACRO



; *************************************************************************************************
; Copies a 16-bit value from a ZP pointer list by Y.
; Incoming:
;   destination     Address, The destination.
;   sourcelist      ZP Address, The source ZPY list.
; Incoming:
;   Y is the offset in the list to use.
; Outgoing:
;   A is the high byte stored.
;   Y is Y + 1
;         ABS       ZP
; Size:   11        9
; Cycles: 20 to 22  18 to 20
.MACRO CopyFromZPYList16 destination, sourcelist
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF

	LDA (sourcelist), Y
	STA destination
	INY
	LDA (sourcelist), Y
	STA destination+1
.ENDMACRO



; *************************************************************************************************
; Copies a 16-bit value from a ZP pointer list by Y.
; Incoming:
;   destlist        ZP Address, The destination ZPY list.
;   source          Address, The source.
; Incoming:
;   Y is the offset in the list to use.
; Outgoing:
;   A is the high byte stored.
;   Y is Y + 1
;         ABS  ZP
; Size:   11   9
; Cycles: 22   20
.MACRO CopyToZPYList16 destlist, source
	.IF .MATCH(.LEFT(1, {source}), #)
		.ERROR "Source is an immediate."
	.ENDIF

	LDA source
	STA (destlist), Y
	INY
	LDA source+1
	STA (destlist), Y
.ENDMACRO



; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Storing Values

; *************************************************************************************************
; Stores an 8-bit immediate value at the destination address.
; Incoming:
;   destination     Address, The destination.
;   value           Byte, The immediate value.
; Outgoing:
;   A is #value.
;         ABS  ZP
; Size:   5    4
; Cycles: 6    5
.MACRO StoreImmediate destination, value
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF

	.IF .MATCH(.LEFT(1, {value}), #)
	  LDA value
	.ELSE
		.ERROR "Value is not an immediate."
	.ENDIF
  STA destination
.ENDMACRO



; *************************************************************************************************
; Stores an 8-bit immediate value at the destination address by X.
; Incoming:
;   destination     Address, The destination.
;   value           Byte, The immediate value.
; Outgoing:
;   X is #value.
;         ABS  ZP
; Size:   5    4
; Cycles: 6    5
.MACRO StoreImmediateByX destination, value
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF

	.IF .MATCH(.LEFT(1, {value}), #)
	  LDX value
	.ELSE
		.ERROR "Value is not an immediate."
	.ENDIF
  STX destination
.ENDMACRO



; *************************************************************************************************
; Stores an 8-bit immediate value at the destination address by Y.
; Incoming:
;   destination     Address, The destination.
;   value           Byte, The immediate value.
; Outgoing:
;   Y is #value.
;         ABS  ZP
; Size:   5    4
; Cycles: 6    5
.MACRO StoreImmediateByY destination, value
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF

	.IF .MATCH(.LEFT(1, {value}), #)
	  LDY value
	.ELSE
		.ERROR "Value is not an immediate."
	.ENDIF
  STY destination
.ENDMACRO



; *************************************************************************************************
; Stores a 16-bit little endian immediate value at the destination address.
; Incoming:
;   destination     Address, The destination.
;   value           Word, The immediate value.
; Outgoing:
;   A is #>(value).
;         ABS  ZP
; Size:   10   8
; Cycles: 12   10
.MACRO StoreImmediate16 destination, value
	.IF .MATCH(.LEFT(1, {value}), #)
  	LDA #.LOBYTE(.RIGHT(.TCOUNT({value})-1, {value}))
  .ELSE
		.ERROR "Value is not an immediate."
  .ENDIF
  STA destination

 	LDA #.HIBYTE(.RIGHT(.TCOUNT({value})-1, {value}))
  STA destination+1
.ENDMACRO



; *************************************************************************************************
; Stores a 16-bit little endian immediate value at the destination address by X.
; Incoming:
;   destination     Address, The destination.
;   value           Word, The immediate value.
; Outgoing:
;   X is #>(value).
;         ABS  ZP
; Size:   10   8
; Cycles: 12   10
.MACRO StoreImmediate16ByX destination, value
	.IF .MATCH(.LEFT(1, {value}), #)
  	LDX #.LOBYTE(.RIGHT(.TCOUNT({value})-1, {value}))
  .ELSE
		.ERROR "Value is not an immediate."
  .ENDIF
  STX destination

 	LDX #.HIBYTE(.RIGHT(.TCOUNT({value})-1, {value}))
  STX destination+1
.ENDMACRO



; *************************************************************************************************
; Stores a 16-bit little endian immediate value at the destination address by Y.
; Incoming:
;   destination     Address, The destination.
;   value           Word, The immediate value.
; Outgoing:
;   Y is #>(value).
;         ABS  ZP
; Size:   10   8
; Cycles: 12   10
.MACRO StoreImmediate16ByY destination, value
	.IF .MATCH(.LEFT(1, {value}), #)
  	LDY #.LOBYTE(.RIGHT(.TCOUNT({value})-1, {value}))
  .ELSE
		.ERROR "Value is not an immediate."
  .ENDIF
  STY destination

 	LDY #.HIBYTE(.RIGHT(.TCOUNT({value})-1, {value}))
  STY destination+1
.ENDMACRO



; *************************************************************************************************
; Stores a 16-bit little endian immediate value at the destination address by X & Y (X High).
; Incoming:
;   destination     Address, The destination.
;   value           Word, The immediate value.
; Outgoing:
;   X & Y (X High) is value.
;         ABS  ZP
; Size:   10   8
; Cycles: 12   10
.MACRO StoreImmediate16ByXY destination, value
	LoadXYImmediate value
	StoreXY destination
.ENDMACRO



; *************************************************************************************************
; Stores X and Y to the specified 16-bit little endian value. X is the high byte and Y is the low
; byte.
; Incoming:
;   destination     Address, The destination.
; Incoming:
;   X is the high byte.
;   Y is the low byte.
;         ABS  ZP
; Size:   6    4
; Cycles: 8    6
.MACRO StoreXY destination
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF

	STY destination
	STX destination+1
.ENDMACRO



; *************************************************************************************************
; Sets multiple values to a single immediate value.
; Incoming:
;   value           Byte, The immediate value to use.
;   d[x]            Addresses, The destinations.
; Outgoing:
;   A is #value
; Size:   2 + (# of ZP addresses * 2) + (# of ABS addresses * 3)
; Cycles: 2 + (# of ZP addresses * 3) + (# of ABS addresses * 4)
.MACRO SetMultiple value, d0, d1, d2, d3, d4, d5, d6, d7, d8, d9, d10, d11, d12, d13, d14, d15
	.IF .MATCH(.LEFT(1, {value}), #)
	  LDA #.LOBYTE(.RIGHT(.TCOUNT({value}) - 1, {value}))
	.ELSE
		.ERROR "Value is not an immediate."
	  ;LDA #.LOBYTE(value)
	.ENDIF
	.IFNBLANK d0
		STA d0
	.ENDIF
	.IFNBLANK d1
		STA d1
	.ENDIF
	.IFNBLANK d2
		STA d2
	.ENDIF
	.IFNBLANK d3
		STA d3
	.ENDIF
	.IFNBLANK d4
		STA d4
	.ENDIF
	.IFNBLANK d5
		STA d5
	.ENDIF
	.IFNBLANK d6
		STA d6
	.ENDIF
	.IFNBLANK d7
		STA d7
	.ENDIF
	.IFNBLANK d8
		STA d8
	.ENDIF
	.IFNBLANK d9
		STA d9
	.ENDIF
	.IFNBLANK d10
		STA d10
	.ENDIF
	.IFNBLANK d11
		STA d11
	.ENDIF
	.IFNBLANK d12
		STA d12
	.ENDIF
	.IFNBLANK d13
		STA d13
	.ENDIF
	.IFNBLANK d14
		STA d14
	.ENDIF
	.IFNBLANK d15
		STA d15
	.ENDIF
.ENDMACRO



; *************************************************************************************************
; Sets multiple values to a single 16-bit immediate value.
; Incoming:
;   value           Word, The immediate value to use.
;   d[x]            Addresses, The destinations.
; Outgoing:
;   A is the high byte of value
; Size:   4 + (# of ZP addresses * 4) + (# of ABS addresses * 6)
; Cycles: 4 + (# of ZP addresses * 6) + (# of ABS addresses * 8)
.MACRO SetMultiple16 value, d0, d1, d2, d3, d4, d5, d6, d7, d8, d9, d10, d11, d12, d13, d14, d15
	; Set the low byte ==============================================================================
	.IF .MATCH(.LEFT(1, {value}), #)
	  LDA #.LOBYTE(.RIGHT(.TCOUNT({value}) - 1, {value}))
	.ELSE
		.ERROR "Value is not an immediate."
	  ;LDA #.LOBYTE(value)
	.ENDIF
	.IFNBLANK d0
		STA d0
	.ENDIF
	.IFNBLANK d1
		STA d1
	.ENDIF
	.IFNBLANK d2
		STA d2
	.ENDIF
	.IFNBLANK d3
		STA d3
	.ENDIF
	.IFNBLANK d4
		STA d4
	.ENDIF
	.IFNBLANK d5
		STA d5
	.ENDIF
	.IFNBLANK d6
		STA d6
	.ENDIF
	.IFNBLANK d7
		STA d7
	.ENDIF
	.IFNBLANK d8
		STA d8
	.ENDIF
	.IFNBLANK d9
		STA d9
	.ENDIF
	.IFNBLANK d10
		STA d10
	.ENDIF
	.IFNBLANK d11
		STA d11
	.ENDIF
	.IFNBLANK d12
		STA d12
	.ENDIF
	.IFNBLANK d13
		STA d13
	.ENDIF
	.IFNBLANK d14
		STA d14
	.ENDIF
	.IFNBLANK d15
		STA d15
	.ENDIF


	; Set the high byte =============================================================================
	.IF .MATCH(.LEFT(1, {value}), #)
  	LDA #.HIBYTE(.RIGHT(.TCOUNT({value}) - 1, {value}))
	.ELSE
  	LDA #.HIBYTE(value)
	.ENDIF
	.IFNBLANK d0
		STA d0+1
	.ENDIF
	.IFNBLANK d1
		STA d1+1
	.ENDIF
	.IFNBLANK d2
		STA d2+1
	.ENDIF
	.IFNBLANK d3
		STA d3+1
	.ENDIF
	.IFNBLANK d4
		STA d4+1
	.ENDIF
	.IFNBLANK d5
		STA d5+1
	.ENDIF
	.IFNBLANK d6
		STA d6+1
	.ENDIF
	.IFNBLANK d7
		STA d7+1
	.ENDIF
	.IFNBLANK d8
		STA d8+1
	.ENDIF
	.IFNBLANK d9
		STA d9+1
	.ENDIF
	.IFNBLANK d10
		STA d10+1
	.ENDIF
	.IFNBLANK d11
		STA d11+1
	.ENDIF
	.IFNBLANK d12
		STA d12+1
	.ENDIF
	.IFNBLANK d13
		STA d13+1
	.ENDIF
	.IFNBLANK d14
		STA d14+1
	.ENDIF
	.IFNBLANK d15
		STA d15+1
	.ENDIF
.ENDMACRO



; *************************************************************************************************
; Clears multiple values. This is SetMultiple with an automatic 0. Supports up to 16 destinations.
; Incoming:
;   d[x]         Addresses, The destinations.
; Outgoing:
;   A is 0
; Size:   2 + (# of ZP addresses * 2) + (# of ABS addresses * 3)
; Cycles: 2 + (# of ZP addresses * 3) + (# of ABS addresses * 4)
.MACRO ClearMultiple d0, d1, d2, d3, d4, d5, d6, d7, d8, d9, d10, d11, d12, d13, d14, d15
	SetMultiple #0, d0, d1, d2, d3, d4, d5, d6, d7, d8, d9, d10, d11, d12, d13, d14, d15
.ENDMACRO



; *************************************************************************************************
; Clears multiple 16-bit values. This is SetMultiple16 with an automatic 0. Supports up to 16 destinations.
; Incoming:
;   d[x]            Addresses, The destinations.
; Outgoing:
;   A is 0
; Size:   2 + (# of ZP addresses * 4) + (# of ABS addresses * 6)
; Cycles: 2 + (# of ZP addresses * 6) + (# of ABS addresses * 8)
.MACRO ClearMultiple16 d0, d1, d2, d3, d4, d5, d6, d7, d8, d9, d10, d11, d12, d13, d14, d15
	LDA #0
	.IFNBLANK d0
		STA d0
		STA d0+1
	.ENDIF
	.IFNBLANK d1
		STA d1
		STA d1+1
	.ENDIF
	.IFNBLANK d2
		STA d2
		STA d2+1
	.ENDIF
	.IFNBLANK d3
		STA d3
		STA d3+1
	.ENDIF
	.IFNBLANK d4
		STA d4
		STA d4+1
	.ENDIF
	.IFNBLANK d5
		STA d5
		STA d5+1
	.ENDIF
	.IFNBLANK d6
		STA d6
		STA d6+1
	.ENDIF
	.IFNBLANK d7
		STA d7
		STA d7+1
	.ENDIF
	.IFNBLANK d8
		STA d8
		STA d8+1
	.ENDIF
	.IFNBLANK d9
		STA d9
		STA d9+1
	.ENDIF
	.IFNBLANK d10
		STA d10
		STA d10+1
	.ENDIF
	.IFNBLANK d11
		STA d11
		STA d11+1
	.ENDIF
	.IFNBLANK d12
		STA d12
		STA d12+1
	.ENDIF
	.IFNBLANK d13
		STA d13
		STA d13+1
	.ENDIF
	.IFNBLANK d14
		STA d14
		STA d14+1
	.ENDIF
	.IFNBLANK d15
		STA d15
		STA d15+1
	.ENDIF
.ENDMACRO



; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Loading Values

; *************************************************************************************************
; Loads Y with the low byte then X with the high byte of an immediate 16-bit value.
; Incoming:
;   value           Word, The immediate value.
; Outgoing:
;   X is the high byte of the immediate value.
;   Y is the low byte of the immediate value.
; Size:   4
; Cycles: 4
.MACRO LoadXYImmediate value
	.IF .MATCH(.LEFT(1, {value}), #)
	  LDY #.LOBYTE(.RIGHT(.TCOUNT({value}) - 1, {value}))
	  LDX #.HIBYTE(.RIGHT(.TCOUNT({value}) - 1, {value}))
  .ELSE
		.ERROR "Value is not an immediate."
	  ;LDY #.LOBYTE(value)
	  ;LDX #.HIBYTE(value)
  .ENDIF
.ENDMACRO



; *************************************************************************************************
; Loads X and Y from the specified 16-bit little endian value. X is the high byte and Y is the low
; byte.
; Incoming:
;   source          Address, The value to use.
; Outgoing:
;   X is the high byte.
;   Y is the low byte.
;         ABS  ZP
; Size:   6    4
; Cycles: 8    6
.MACRO LoadXY destination
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF

	LDY destination
	LDX destination+1
.ENDMACRO



; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Math Operations

; *@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*
; *@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*
; Add/Subtract Immediates

; *************************************************************************************************
; Adds an 8-bit immediate value to an 8-bit value.
; Incoming:
;   destination     Address, The value to modify.
;   value           Byte, The immediate value.
; Outgoing:
;   A is destination after the add.
;         ABS  ZP
; Size:   9    7
; Cycles: 12   10
.MACRO AddImmediate destination, value
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF

	CLC
	LDA destination
  .IF .MATCH(.LEFT(1, {value}), #)
  	ADC value
  .ELSE
		.ERROR "Value is not an immediate."
  	;ADC #value
  .ENDIF
  STA destination
.ENDMACRO



; *************************************************************************************************
; Adds an 8-bit immediate value to a 16-bit little endian value.
; Incoming:
;   destination     Address, The value to modify.
;   value           Byte, The immediate value.
; Outgoing:
;   A is the low byte of destination after the add.
;         ABS       ZP
; Size:   14        11
; Cycles: 15 to 18  13 to 15
.MACRO AddImmediate8To16 destination, value
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF

	.LOCAL @SkipHighByte
  CLC
  LDA destination
  .IF .MATCH(.LEFT(1, {value}), #)
  	ADC value
  .ELSE
		.ERROR "Value is not an immediate."
  	;ADC #value
  .ENDIF
  STA destination
  BCC @SkipHighByte
  INC destination+1
@SkipHighByte:
.ENDMACRO



; *************************************************************************************************
; Adds a 16-bit immediate value to a 16-bit little endian value.
; Incoming:
;   destination     Address, The value to modify.
;   value           Word, The immediate value.
; Outgoing:
;   A is the high byte after the add.
;         ABS  ZP
; Size:   17   13
; Cycles: 22   18
.MACRO AddImmediate16 destination, value
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF

	; Low byte
  CLC
  LDA destination
	.IF .MATCH(.LEFT(1, {value}), #)
  	ADC #.LOBYTE(.RIGHT(.TCOUNT({value})-1, {value}))
  .ELSE
		.ERROR "Value is not an immediate."
  	;ADC #.LOBYTE(value)
  .ENDIF
  STA destination

	; High byte
	LDA destination+1
	.IF .MATCH(.LEFT(1, {value}), #)
  	ADC #.HIBYTE(.RIGHT(.TCOUNT({value})-1, {value}))
  .ELSE
		.ERROR "Value is not an immediate."
  	;ADC #.HIBYTE(value)
  .ENDIF
  STA destination+1
.ENDMACRO



; *************************************************************************************************
; Adds an 8-bit value to another value.
; Destination += Source
; Incoming:
;   destination     Address, The destination.
;   source          Address, The source.
; Outgoing:
;   A is destination.
;         ABS += ABS  ABS += ZP  ZP += ABS  ZP += ZP
; Size:   10          9          8          7
; Cycles: 14          13         12         11
.MACRO AddValue destination, source
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF

	.IF .MATCH(.LEFT(1, {source}), #)
		.ERROR "Source is an immediate."
	.ENDIF
	CLC
  LDA destination
  ADC source
  STA destination
.ENDMACRO



; *************************************************************************************************
; Adds an 8-bit value to a 16-bit little endian value.
; Incoming:
;   destination     Address, The value to modify.
;   source          Byte, The source.
; Outgoing:
;   A is the low byte of destination after the add.
;         ABS += ABS  ABS += ZP  ZP += ABS  ZP += ZP
; Size:   15          14         12         11
; Cycles: 17 to 22    16 to 21   15 to 19   14 to 18
.MACRO AddValue8To16 destination, source
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF
	.IF .MATCH(.LEFT(1, {source}), #)
		.ERROR "Source is an immediate."
	.ENDIF

	.LOCAL @SkipHighByte
  CLC
  LDA destination
	ADC source
  STA destination
  BCC @SkipHighByte
  INC destination+1
@SkipHighByte:
.ENDMACRO



; *************************************************************************************************
; Adds a 16-bit value to another value.
; Destination += Source
; Incoming:
;   destination     Address, The destination.
;   source          Address, The source.
; Outgoing:
;   A is the high byte of destination.
;         ABS += ABS  ABS += ZP  ZP += ABS  ZP += ZP
; Size:   19          17         15         13
; Cycles: 26          24         22         20
.MACRO AddValue16 destination, source
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF
	.IF .MATCH(.LEFT(1, {source}), #)
		.ERROR "Source is an immediate."
	.ENDIF

	CLC
  LDA destination
  ADC source
  STA destination
  LDA destination+1
  ADC source+1
  STA destination+1
.ENDMACRO



; *************************************************************************************************
; Increments a 16-bit little endian value.
; Incoming:
;   destination     Address, The value to modify.
; Outgoing: All registers are preserved.
;         ABS      ZP
; Size:   8        6
; Cycles: 9 to 14  8 to 12
.MACRO Increment16 destination
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF

	.LOCAL @SkipHighByte
  INC destination
  BNE @SkipHighByte
  INC destination+1
@SkipHighByte:
.ENDMACRO



; *************************************************************************************************
; Increments a 32-bit little endian value.
; Incoming:
;   destination     Address, The value to modify.
; Outgoing: All registers are preserved.
;         ABS      ZP
; Size:   18       14
; Cycles: 9 to 30  8 to 26
.MACRO Increment32 destination
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF

	.LOCAL @SkipHigherBytes
  INC destination
  BNE @SkipHigherBytes
  INC destination+1
  BNE @SkipHigherBytes
  INC destination+2
  BNE @SkipHigherBytes
  INC destination+3
@SkipHigherBytes:
.ENDMACRO



; *************************************************************************************************
; Subtracts an 8-bit immediate value from an 8-bit value.
; Incoming:
;   destination     Address, The value to modify.
;   value           Byte, The immediate value.
; Outgoing:
;   A is destination after the subtract.
;         ABS  ZP
; Size:   9    7
; Cycles: 12   10
.MACRO SubtractImmediate destination, value
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF

	SEC
	LDA destination
  .IF .MATCH(.LEFT(1, {value}), #)
  	SBC value
  .ELSE
		.ERROR "Value is not an immediate."
  	;SBC #value
  .ENDIF
  STA destination
.ENDMACRO



; *************************************************************************************************
; Subtracts an 8-bit immediate value from a 16-bit little endian value.
; Incoming:
;   destination     Address, The value to modify.
;   value           Byte, The immediate value.
; Outgoing:
;   A is the low byte of destination after the subtract.
;         ABS       ZP
; Size:   14        11
; Cycles: 15 to 18  13 to 15
.MACRO SubtractImmediate8From16 destination, value
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF

	.LOCAL @SkipHighByte
  SEC
  LDA destination
  .IF .MATCH(.LEFT(1, {value}), #)
  	SBC value
  .ELSE
		.ERROR "Value is not an immediate."
  	;SBC #value
  .ENDIF
  STA destination
  BCS @SkipHighByte
  DEC destination+1
@SkipHighByte:
.ENDMACRO



; *************************************************************************************************
; Subtracts a 16-bit immediate value from a 16-bit little endian value.
; Incoming:
;   destination     Address, The value to modify.
;   value           Word, The immediate value.
; Outgoing:
;   A is the high byte after the subtraction.
;         ABS   ZP
; Size:   17    13
; Cycles: 22    18
.MACRO SubtractImmediate16 destination, value
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF

	; Low byte
  SEC
  LDA destination
	.IF .MATCH(.LEFT(1, {value}), #)
  	SBC #.LOBYTE(.RIGHT(.TCOUNT({value})-1, {value}))
  .ELSE
		.ERROR "Value is not an immediate."
  	;SBC #.LOBYTE(value)
  .ENDIF
  STA destination

	; High byte
	LDA destination+1
	.IF .MATCH(.LEFT(1, {value}), #)
  	SBC #.HIBYTE(.RIGHT(.TCOUNT({value})-1, {value}))
  .ELSE
		.ERROR "Value is not an immediate."
  	;SBC #.HIBYTE(value)
  .ENDIF
  STA destination+1
.ENDMACRO



; *************************************************************************************************
; Subtracts an 8-bit value from another value.
; Destination -= Source
; Incoming:
;   destination     Address, The destination.
;   source          Address, The source.
; Outgoing:
;   A is destination.
;         ABS -= ABS  ABS -= ZP  ZP -= ABS  ZP -= ZP
; Size:   10          9          8          7
; Cycles: 14          13         12         11
.MACRO SubtractValue destination, source
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF
	.IF .MATCH(.LEFT(1, {source}), #)
		.ERROR "Source is an immediate."
	.ENDIF

	SEC
  LDA destination
  SBC source
  STA destination
.ENDMACRO



; *************************************************************************************************
; Subtracts an 8-bit value from a 16-bit little endian value.
; Incoming:
;   destination     Address, The value to modify.
;   value           Byte, The value to subtract from destination.
; Outgoing:
;   A is the low byte of destination after the subtract.
;         ABS -= ABS  ABS -= ZP  ZP -= ABS  ZP -= ZP
; Size:   15          14         12         11
; Cycles: 17 to 22    16 to 21   15 to 19   14 to 18
.MACRO SubtractValue8From16 destination, source
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF
	.IF .MATCH(.LEFT(1, {source}), #)
		.ERROR "Source is an immediate."
	.ENDIF

	.LOCAL @SkipHighByte
  SEC
  LDA destination
	SBC source
  STA destination
  BCS @SkipHighByte
  DEC destination+1
@SkipHighByte:
.ENDMACRO



; *************************************************************************************************
; Subtracts a 16-bit value from another value.
; Destination -= Source
; Incoming:
;   destination     Address, The destination.
;   source          Address, The source.
; Outgoing:
;   A is the high byte of destination.
;         ABS += ABS  ABS += ZP  ZP += ABS  ZP += ZP
; Size:   19          17         15         13
; Cycles: 26          24         22         20
.MACRO SubtractValue16 destination, source
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF
	.IF .MATCH(.LEFT(1, {source}), #)
		.ERROR "Source is an immediate."
	.ENDIF

	SEC
  LDA destination
  SBC source
  STA destination
  LDA destination+1
  SBC source+1
  STA destination+1
.ENDMACRO



; *************************************************************************************************
; Decrements a 16-bit little endian value.
; WARNING! A IS LOST UNLIKE INCREMENT16!
; Incoming:
;   destination     Address, The value to modify.
; Outgoing:
;   A is the low byte of destination before the decrement.
;         ABS       ZP
; Size:   11        8
; Cycles: 13 to 18  11 to 15
.MACRO Decrement16 destination
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF

	.LOCAL @SkipHighByte
	LDA destination
	BNE @SkipHighByte
	DEC destination+1
@SkipHighByte:
	DEC destination
.ENDMACRO



; *@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*
; *@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*
; Add/Subtract To/From a list by zero page Y

; *************************************************************************************************
; Adds a zero page pointer list value (using Y) to an address.
; Destination += Source
; Incoming:
;   destination     Address, The destination.
;   sourcelist      ZP Address, The source ZPY list.
; Incoming:
;   Y is the offset into source to use.
; Outgoing:
;   A is the value
;         ABS       ZP
; Size:   9         7
; Cycles: 15 to 16  13 to 14
.MACRO AddFromZPYList destination, sourcelist
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF

  CLC
  LDA destination
  ADC (sourcelist),Y
  STA destination
.ENDMACRO



; *************************************************************************************************
; Adds an address value to a zero page pointer list value (using Y).
; Destination += Source
; Incoming:
;   destlist        ZP Address, The destination ZPY list.
;   source          Address, The source.
; Incoming:
;   Y is the offset into source to use.
; Outgoing:
;   A is the value.
;         ABS       ZP
; Size:   8         7
; Cycles: 17 to 18  16 to 17
.MACRO AddToZPYList destlist, source
	.IF .MATCH(.LEFT(1, {source}), #)
		.ERROR "Source is an immediate."
	.ENDIF

  CLC
  LDA (destlist),Y
  ADC source
  STA (destlist),Y
.ENDMACRO



; *************************************************************************************************
; Adds a 16-bit zero page pointer list value (using Y) to an address.
; Destination += Source
; Incoming:
;   destination     Address, The destination.
;   sourcelist      ZP Address, The source ZPY list.
; Incoming:
;   Y is the offset into source to use.
; Outgoing:
;   A is the high byte
;   Y is Y + 1
;         ABS       ZP
; Size:   18        14
; Cycles: 30 to 32  26 to 28
.MACRO AddFromZPYList16 destination, sourcelist
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF

  CLC
  LDA destination
  ADC (sourcelist),Y
  STA destination
  INY
  LDA destination+1
  ADC (sourcelist),Y
  STA destination+1
.ENDMACRO



; *************************************************************************************************
; Subtracts a 16-bit zero page pointer list value (using Y) from an address.
; Destination -= Source
; Incoming:
;   destination     Address, The destination address.
;   source          ZP Address, The source.
; Incoming:
;   Y is the offset into source to use.
; Outgoing:
;   A is the high byte
;   Y is Y + 1
;         ABS       ZP
; Size:   18        14
; Cycles: 30 to 32  26 to 28
.MACRO SubtractByZPYList16 destination, source
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF

  SEC
  LDA destination
  SBC (source),Y
  STA destination
  INY
  LDA destination+1
  SBC (source),Y
  STA destination+1
.ENDMACRO



; *@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*
; *@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*@*
; Misc

; *************************************************************************************************
; Converts an 8-bit value to its negative.
; Incoming:
;   destination     Address, The destination address.
; Outgoing:
;   A is lost
;         ABS  ZP
; Size:   11   8
; Cycles: 16   13
.MACRO Negative destination
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF

	LDA destination
	EOR #$FF
	STA destination
	INC destination
.ENDMACRO



; *************************************************************************************************
; Converts a 16-bit value to its negative.
; Incoming:
;   destination     Address, The destination address.
; Outgoing:
;   A is lost
;         ABS       ZP
; Size:   24        18
; Cycles: 29 to 34  24 to 28
.MACRO Negative16 destination
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF

	; Flip the bits
	LDA destination
	EOR #$FF
	STA destination
	LDA destination+1
	EOR #$FF
	STA destination+1

	; Increment it
	Increment16 destination
.ENDMACRO



; *************************************************************************************************
; Converts a 32-bit value to its negative.
; This is so large, I recommend using a subroutine instead.
; Incoming:
;   destination     Address, The destination address.
; Outgoing:
;   A is lost
;         ABS       ZP
; Size:   54        38
; Cycles: 49 to 70  40 to 58
.MACRO Negative32 destination
	.IF .MATCH(.LEFT(1, {destination}), #)
		.ERROR "Destination is an immediate."
	.ENDIF

	; Flip the bits
	.REPEAT 4, Index
		LDA destination + Index
		EOR #$FF
		STA destination + Index
	.ENDREPEAT

	; Increment it
	Increment32 destination
.ENDMACRO



; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Misc Macros

; *************************************************************************************************
; This ignores the next word (to ignore 2 1-byte or 1 2-byte instruction).
; This currently works by using BIT $abcd, which will do a read and modify the flags, but changes
; nothing otherwise.
; Outgoing:
;   NVZ flags lost.
; Size:   1
; Cycles: 4
.MACRO IgnoreNextWord
	.BYTE OPCODE::BIT_Abs
.ENDMACRO



; *************************************************************************************************
; This ignores the next byte (to ignore a 1-byte instruction).
; This currently works by using BIT $ab, which will do a read and modify the flags, but changes
; nothing otherwise.
; Outgoing:
;   NVZ flags lost.
; Size:   1
; Cycles: 3
.MACRO IgnoreNextByte
	.BYTE OPCODE::BIT_ZP
.ENDMACRO




; #################################################################################################
; #################################################################################################
; #################################################################################################
; Data Macros

; *************************************************************************************************
; Creates a PASCAL-style string which starts with a byte length first.
; Incoming:
;   value           String, A single string value.
;   ...             Up to 16 additional values to add in .BYTE format.
; Size: STRLEN(value) + 1
.MACRO PString value, a0, b0, c0, d0, e0, f0, g0, h0, i0, j0, k0, l0, m0, n0, o0, p0
	.LOCAL PStringStart, PStringEnd
	.BYTE PStringEnd - PStringStart
PStringStart:
	.BYTE value
	.IFNBLANK a0
		.BYTE a0
	.ENDIF
	.IFNBLANK b0
		.BYTE b0
	.ENDIF
	.IFNBLANK c0
		.BYTE c0
	.ENDIF
	.IFNBLANK d0
		.BYTE d0
	.ENDIF
	.IFNBLANK e0
		.BYTE e0
	.ENDIF
	.IFNBLANK f0
		.BYTE f0
	.ENDIF
	.IFNBLANK g0
		.BYTE g0
	.ENDIF
	.IFNBLANK h0
		.BYTE h0
	.ENDIF
	.IFNBLANK i0
		.BYTE i0
	.ENDIF
	.IFNBLANK j0
		.BYTE j0
	.ENDIF
	.IFNBLANK k0
		.BYTE k0
	.ENDIF
	.IFNBLANK l0
		.BYTE l0
	.ENDIF
	.IFNBLANK m0
		.BYTE m0
	.ENDIF
	.IFNBLANK n0
		.BYTE n0
	.ENDIF
	.IFNBLANK o0
		.BYTE o0
	.ENDIF
	.IFNBLANK p0
		.BYTE p0
	.ENDIF
PStringEnd:
.ENDMACRO



; *************************************************************************************************
; Creates a PASCAL-style string which starts with a byte length first, allowing escapes.
; Escapes allowed at this time:
;   ~     Toggles highlight state.
; Incoming:
;   value           String, A single string value.
; Size: STRLEN(value) + 1
PStringEscape_Highlight .SET 0
.MACRO PStringEscape value
	.LOCAL PStringStart, PStringEnd, char
	PStringEscape_Highlight .SET 0
	.BYTE PStringEnd - PStringStart
PStringStart:
	.REPEAT .STRLEN(value), index
		char .SET .STRAT(value, index)
		PStringEscapeInnerProcess char
	.ENDREPEAT
PStringEnd:
.ENDMACRO
; =*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=
; Processes a single character for PStringEscape
.MACRO PStringEscapeInnerProcess char
	.IF char = '~'
		PStringEscape_Highlight .SET PStringEscape_Highlight ^ 1
	.ELSEIF PStringEscape_Highlight > 0
		.BYTE char | $80
	.ELSE
		.BYTE char
	.ENDIF
.ENDMACRO



; *************************************************************************************************
; Creates a PASCAL-style string which starts with a word length first.
; Incoming:
;   value           String, A single string value.
;   ...             Up to 16 additional values to add in .BYTE format.
; Size: STRLEN(value) + 1
.MACRO PString16 value, a0, b0, c0, d0, e0, f0, g0, h0, i0, j0, k0, l0, m0, n0, o0, p0
	.LOCAL PString16Start, PString16End
	.WORD PString16End - PString16Start
PString16Start:
	.BYTE value
	.IFNBLANK a0
		.BYTE a0
	.ENDIF
	.IFNBLANK b0
		.BYTE b0
	.ENDIF
	.IFNBLANK c0
		.BYTE c0
	.ENDIF
	.IFNBLANK d0
		.BYTE d0
	.ENDIF
	.IFNBLANK e0
		.BYTE e0
	.ENDIF
	.IFNBLANK f0
		.BYTE f0
	.ENDIF
	.IFNBLANK g0
		.BYTE g0
	.ENDIF
	.IFNBLANK h0
		.BYTE h0
	.ENDIF
	.IFNBLANK i0
		.BYTE i0
	.ENDIF
	.IFNBLANK j0
		.BYTE j0
	.ENDIF
	.IFNBLANK k0
		.BYTE k0
	.ENDIF
	.IFNBLANK l0
		.BYTE l0
	.ENDIF
	.IFNBLANK m0
		.BYTE m0
	.ENDIF
	.IFNBLANK n0
		.BYTE n0
	.ENDIF
	.IFNBLANK o0
		.BYTE o0
	.ENDIF
	.IFNBLANK p0
		.BYTE p0
	.ENDIF
PString16End:
.ENDMACRO


; If we need them:
; From https://archive.nes.science/nesdev-forums/f2/t16314.xhtml
;.DEFINE StringStartsWith(string, teststring) .XMATCH(.SPRINTF(.SPRINTF("%%.%ds", .STRLEN(teststring)), string), teststring)
;.DEFINE StringEndsWith(string, teststring) .XMATCH(.SPRINTF(.SPRINTF("%%.%ds%%s", (.STRLEN(teststring) < .STRLEN(string)) * (.STRLEN(string) - .STRLEN(teststring))), string, teststring), string)

.ENDIF ; .IFNDEF IncludeFile_6502
