; Atari OS values.
; License: GPL-2, see LICENSE
; Date Created: 2021-08-01
; (c) 2021 to 2022 Mark Ormston <memso@memso.net>

.IFNDEF IncludeFile_AtariOS
IncludeFile_AtariOS = 1
.INCLUDE "AtariSystem.inc"

; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Addresses
; See "Mapping the Atari", section 103 and 212 for general details.
.SCOPE AtariOS
	; ***********************************************************************************************
	; OS Zero Page
	; These are all prefixed with ZP_ to make it really obvious they're zero page addresses.

	; $00 - $0F =====================================================================================

	; Word, Linbug storage. Sometimes used to store the VBLANK timer.
	ZP_LINZBS    = $00
	; Byte, XL+ only. No idea what it's for.
	ZP_LNFLG     = $00
	; Byte, XL+ only. If zero, memory has issues during power up.
	ZP_NGFLAG    = $01

	; Address, Cassette initialization location. Seems to be used for boot location as well in many
	; instances.
	ZP_CASINI    = $02

	; Word, RAM pointer for the memory test.
	ZP_RAMLO     = $04

	; Byte, Temporary location for ram size during power up, then moved to RAMTOP.
	ZP_TRAMSZ    = $06

	; Byte, RAM test data. 1 if the B/right cartridge is used.
	ZP_TSTDAT    = $07
	; Byte, XL+ only. Command flag for 835/1030 modems.
	ZP_CMCMD     = $07

	; Byte, 0 while powering up, $FF otherwise.
	ZP_WARMST    = $08

	; Byte, The boot success flag.
	ZP_BOOTQ     = $09

	; Address, Start vector for non-cartridge software, for DOS.
	ZP_DOSVEC    = $0A

	; Address, Initialization address for the disk boot.
	ZP_DOSINI    = $0C

	; Address, The maximum address allowed for the current program. Also used by BASIC.
	ZP_APPMHI    = $0E


	; $10 - $1F =====================================================================================

	; Byte, Start of OS RAM clear location.
	ZP_INTZBS    = $10
	; Byte, POKEY Mask. Shadow for POKEY_W::IRQEN.
	ZP_POKMSK    = $10

	; Byte, Break key flag. 0 means the BREAK key is being pressed. Anything else means it is not.
	ZP_BRKKEY    = $11

	; 24-bit Big Endian, The real time clock value, updated every vertical blank.
	ZP_RTCLOK    = $12
	ZP_RTCLOKH   = $12
	ZP_RTCLOKM   = $13
	ZP_RTCLOKL   = $14

	; Address, Indirect buffer address register. Used by OS disk routines.
	ZP_BUFADR    = $15

	; Byte, The command for the current CIO command.
	ZP_ICCOMT    = $17

	; Address, Disk file manager pointer.
	ZP_DSKFMS    = $18

	; Address, Disk utilities pointer.
	ZP_DSKUTL    = $1A

	; Byte, Printer time out register.
	ZP_PTIMOT    = $1C
	; 4 bytes, OS buffer pointers. Unused.
	ZP_ABUFPT    = $1C

	; Byte, The pointer into the print buffer. From 0 to PBUFSZ-1.
	ZP_PBPNT     = $1D

	; Byte, Size of the print buffer.
	ZP_PBUFSZ    = $1E

	; Byte, Printer temporary register.
	ZP_PTEMP     = $1F


	; $20 - $2F =====================================================================================
	; CIO/IOCB work area.

	; 16 bytes, The IOCB zero page data for the current operation.
	ZP_ZIOCB     = $20
	ZP_IOCBAS    = ZP_ZIOCB
	; Byte, Handler index number. $FF if free.
	ZP_ICHIDZ    = $20

	; Byte, Device/drive number.
	ZP_ICDNOZ    = $21

	; Byte, Command code.
	ZP_ICCOMZ    = $22

	; Byte, Status of the last IOCB action. See ATARIOS_CIO_STATUS::*
	ZP_ICSTAZ    = $23

	; Address, Buffer address.
	ZP_ICBAZ     = $24
	ZP_ICBALZ    = $24
	ZP_ICBAHZ    = $25

	; Address, Put byte routine address - 1 (for use in an RTS).
	ZP_ICPTZ     = $26
	ZP_ICPTLZ    = $26
	ZP_ICPTHZ    = $27

	; Word, Buffer length.
	ZP_ICBLZ     = $28
	ZP_ICBLLZ    = $28
	ZP_ICBLHZ    = $29

	; Byte, Auxiliary 1.
	ZP_ICAX1Z    = $2A

	; Byte, Auxiliary 2.
	ZP_ICAX2Z    = $2B

	; Byte, CIO local use.
	ZP_ICSPRZ    = $2C

	; Byte, The offset into the IOCB data. This is the IOCB channel number * 16.
	ZP_ICIDNO    = $2E

	; Byte, The character being processed for the current CIO operation.
	ZP_CIOCHR    = $2F


	; $30 - $3F =====================================================================================
	; SIO data

	; Byte, Internal status register.
	ZP_STATUS    = $30

	; Byte, Checksum.
	ZP_CHKSUM    = $31

	; Address, Pointer to data buffer.
	ZP_BUFR      = $32
	ZP_BUFRLO    = $32
	ZP_BUFRHI    = $33

	; Address, Next byte past the end of the buffer.
	ZP_BFEN      = $34
	ZP_BFENLO    = $34
	ZP_BFENHI    = $35

	; Byte, Number of command frame retries.
	ZP_CRETRY    = $36
	; Address, XL+ only. Pointer to temporary peripheral handler loader routines.
	ZP_LTEMP     = $36
	; Byte, Number of device retries.
	ZP_DRETRY    = $37

	; Byte, Data buffer full flag. $FF means full.
	ZP_BUFRFL    = $38

	; Byte, Receiving done flag. $FF means done.
	ZP_RECVDN    = $39

	; Byte, Transmit done flag. $FF means done.
	ZP_XMTDON    = $3A

	; Byte, Checksum sent flag. $FF means sent.
	ZP_CHKSNT    = $3B

	; Byte, No checksum follows data flag. $FF means to not include a checksum.
	ZP_NOCKSM    = $3C

	; Byte, Cassette buffer pointer.
	ZP_BPTR      = $3D

	; Byte, Cassette file type by inter-record gap (IRG). Normal gaps are non-zero, and continuous
	; gaps are 0.
	ZP_FTYPE     = $3E

	; Byte, Cassette end of file flag. $FF if the end has been reached.
	ZP_FEOF      = $3F


	; $40 - $4F =====================================================================================

	ZP_FREQ      = $40         ;FREQ COUNTER FOR CONSOLE SPEAKER

	; Byte, 0 for no sound during I/O operations. Non-zero to make loading sounds.
	ZP_SOUNDR    = $41

	; Byte, Set to non-zero if high-speed or critical code is currently in use, such as disk I/O.
	ZP_CRITIC    = $42

	ZP_FMSZPG    = $43         ;DISK FILE MANAGER SYSTEM STORAGE (7 BYTES)

	ZP_CKEY      = $4A         ;SET WHEN GAME START PRESSED
	; Word, XL+ only. Temporary storage register for peripheral handler loader.
	ZP_ZCHAIN    = $4A
	ZP_CASSBT    = $4B         ;CASSETTE BOOT FLAG

	ZP_DSTAT     = $4C         ;DISPLAY STATUS
	ZP_ATRACT    = $4D         ;ATTRACT MODE FLAG
	ZP_DRKMSK    = $4E         ;DARK ATTRACT MASK
	ZP_COLRSH    = $4F         ;ATTRACT COLOR SHIFTER (XOR'D WITH PLAYFIELD)


	; $50 - $5F =====================================================================================

	ZP_TMPCHR    = $50         ;TEMP CHAR STORAGE (DISPLAY HANDLER)
	ZP_HOLD1     = $51         ;TEMP STG (DISPLAY HANDLER)
	ZP_LMARGN    = $52         ;LEFT MARGIN
	ZP_RMARGN    = $53         ;RIGHT MARGIN
	ZP_ROWCRS    = $54         ;CURSOR COUNTERS
	ZP_COLCRS    = $55
	ZP_DINDEX    = $57         ;DISPLAY INDEX (VARIOUS QUANTS)
	ZP_SAVMSC    = $58
	ZP_OLDROW    = $5A         ;PREVIOUS ROW/COL
	ZP_OLDCOL    = $5B
	ZP_OLDCHR    = $5D         ;DATA UNDER CURSOR
	ZP_OLDADR    = $5E


	; $60 - $6F =====================================================================================

	; Address, 1200XL only. Pointer to definition table for F1 to F4 and Shift-F1 to Shift-F4.
	ZP_FKDEF     = $60

	; 2 bytes, The row and column for point to draw to.
	ZP_NEWROW    = $60
	ZP_NEWCOL    = $61

	; Byte, XL+ only. PAL or NTSC flag. 0 for NTSC.
	ZP_PALNTS    = $62

	ZP_LOGCOL    = $63         ;POINTS AT COLUMN IN LOGICAL LINE
	ZP_ADRESS    = $64         ;INDIRECT POINTER
	ZP_MLTTMP    = $66         ;MULTIPLY TEMP
	ZP_OPNTMP    = ZP_MLTTMP   ;FIRST BYTE IS USED IN OPEN AS TEMP
	ZP_SAVADR    = $68
	ZP_RAMTOP    = $6A         ;RAM SIZE DEFINED BY POWER ON LOGIC
	ZP_BUFCNT    = $6B         ;BUFFER COUNT
	ZP_BUFSTR    = $6C         ;EDITOR GETCH POINTER
	ZP_BITMSK    = $6E         ;BIT MASK
	ZP_SHFAMT    = $6F         ;OUTCHR SHIFT


	; $70 - $7F =====================================================================================
	ZP_ROWAC     = $70         ;USED BY "DRAW"
	ZP_COLAC     = $72
	ZP_ENDPT     = $74
	ZP_DELTAR    = $76
	ZP_DELTAC    = $77

	ZP_ROWINC    = $79

	; Address, XL+ Only. Pointer to a keyboard definition table (default: $FB51).
	ZP_KEYDEF    = $79
	ZP_COLINC    = $7A

	ZP_SWPFLG    = $7B         ;NON-0 IF TXT AND RAM SWAPPED
	ZP_HOLDCH    = $7C         ;CH BEFORE CNTL & SHFT PROCESSING IN KGETCH
	ZP_INSDAT    = $7D         ;INSERT CHAR SAVE
	ZP_COUNTR    = $7E         ;DRAW COUNTER


	; Misc ==========================================================================================

	; Address, XL+ Only. Low end of memory available??? (default: $1DFC).
	ZP_LOMEM     = $80

	; 128 bytes, The start of free zero page memory.
	ZP_FREE      = $80



	; ***********************************************************************************************
	; Other Atari OS Addresses

	; Page 2 ========================================================================================

	; 20 address, The OS interrupt address table.
	INTABS       = $200

	; Address, The display list interrupt handler to call.
	VDSLST       = $200

	; Address, Serial proceed line vector.
	VPRCED       = $202

	; Address, Serial interrupt vector.
	VINTER       = $204

	; Address, Break instruction vector.
	VBREAK       = $206

	; Address, Keyboard interrupt vector.
	VKEYBD       = $208

	; Address, Serial input vector.
	VSERIN       = $20A

	; Address, Serial transmit vector.
	VSEROR       = $20C

	; Address, Serial transmit complete vector.
	VSEROC       = $20E

	; Address, Timer 1 vector.
	VTIMR1       = $210

	; Address, Timer 2 vector.
	VTIMR2       = $212

	; Address, Timer 4 vector.
	VTIMR4       = $214

	; Address, Immediate IRQ vector.
	VIMIRQ       = $216

	; Word, Count down timers 1 to 5.
	CDTMV1       = $218
	CDTMV2       = $21A
	CDTMV3       = $21C
	CDTMV4       = $21E
	CDTMV5       = $220

	; Address, Vertical blank immediate.
	VVBLKI       = $222

	; Address, Vertical blank deferred.
	VVBLKD       = $224

	CDTMA1       = $226         ;COUNT DOWN TIMER 1 JSR ADDRESS
	CDTMA2       = $228         ;COUNT DOWN TIMER 2 JSR ADDRESS
	CDTMF3       = $22A         ;COUNT DOWN TIMER 3 FLAG
	SRTIMR       = $22B         ;SOFTWARE REPEAT TIMER
	CDTMF4       = $22C         ;COUNT DOWN TIMER 4 FLAG
	INTEMP       = $22D         ;IAN'S TEMP (???)
	CDTMF5       = $22E         ;COUNT DOWN TIMER 5 FLAG
	SDMCTL       = $22F         ;SAVE DMACTL REGISTER

	; Address, Save display list. As a direct value, or low and hight equivalents.
	SDLST        = $230
	SDLSTL       = $230
	SDLSTH       = $231

	; Byte, SKCTL shadow register.
	SSKCTL       = $232

	; Byte, The light pen horizontal value.
	LPENH        = $234

	; Byte, The light pen vertical value.
	LPENV        = $235

  ; $236 - $239 are unused

	; 4 bytes, The command frame data to pass to SIO. The first is CDEVIC:
	; Byte, The device to use.
	CDEVIC       = $23A

	; Byte, The SIO command to use.
	CCOMND       = $23B

	; Byte, Auxiliary byte 1.
	CAUX1        = $23C

	; Byte, Auxiliary byte 2.
	CAUX2        = $23D

	TEMP         = $23E         ;YES
	ERRFLG       = $23F         ;ERROR FLAG - ANY DEVICE ERROR EXCEPT TIMEOUT

	DFLAGS       = $240         ;DISK FLAGS FROM SECTOR ONE
	DBSECT       = $241         ;NUMBER OF DISK BOOT SECTORS
	BOOTAD       = $242         ;ADDRESS FOR DISK BOOT LOADER
	COLDST       = $244         ;COLDSTART FLAG (1 = DOING COLDSTART)
  ;($245 SPARE)
	DSKTIM       = $246         ;DISK TIME OUT REG
	LINBUF       = $247         ;CHAR LINE BUFFER (40 BYTES)

	GPRIOR       = $26F         ;GLOBAL PRIORITY CELL
	PADDL0       = $270         ;POT 0 SHADOW
	PADDL1       = $271         ;POT 1 SHADOW
	PADDL2       = $272         ;POT 2 SHADOW
	PADDL3       = $273         ;POT 3 SHADOW
	PADDL4       = $274         ;POT 4 SHADOW
	PADDL5       = $275         ;POT 5 SHADOW
	PADDL6       = $276         ;POT 6 SHADOW
	PADDL7       = $277         ;POT 7 SHADOW
	STICK0       = $278         ;JOYSTICK 0 SHADOW
	STICK1       = $279         ;JOYSTICK 1 SHADOW
	STICK2       = $27A         ;JOYSTICK 2 SHADOW
	STICK3       = $27B         ;JOYSTICK 3 SHADOW
	PTRIG0       = $27C         ;PADDLE 0 TRIGGER
	PTRIG1       = $27D         ;PADDLE 1 TRIGGER
	PTRIG2       = $27E         ;PADDLE 2 TRIGGER
	PTRIG3       = $27F         ;PADDLE 3 TRIGGER
	PTRIG4       = $280         ;PADDLE 4 TRIGGER
	PTRIG5       = $281         ;PADDLE 5 TRIGGER
	PTRIG6       = $282         ;PADDLE 6 TRIGGER
	PTRIG7       = $283         ;PADDLE 7 TRIGGER
	STRIG0       = $284         ;JOYSTICK 0 TRIGGER
	STRIG1       = $285         ;JOYSTICK 1 TRIGGER
	STRIG2       = $286         ;JOYSTICK 2 TRIGGER
	STRIG3       = $287         ;JOYSTICK 3 TRIGGER

	CSTAT        = $288         ;(UNUSED)
	WMODE        = $289         ;R/W FLAG FOR CASSETTE
	BLIM         = $28A         ;BUFFER LIMIT (CASSETTE)

  ; $28B - $28F are unused

	TXTROW       = $290         ;TEXT ROWCRS
	TXTCOL       = $291         ;TEXT ROWCOL
	TINDEX       = $293         ;TEXT INDEX
	TXTMSC       = $294         ;FOOLS CONVRT INTO NEW MSC
	TXTOLD       = $296         ;OLDROW & OLDCOL FOR TEXT (AND THEN SOME)
	TMPX1        = $29C
	HOLD3        = $29D
	SUBTMP       = $29E
	HOLD2        = $29F
	DMASK        = $2A0
	TMPLBT       = $2A1
	ESCFLG       = $2A2         ;ESCAPE FLAG
	TABMAP       = $2A3         ;TAB BUFFER
	LOGMAP       = $2B2         ;LOGICAL LINE START BIT MAP
	INVFLG       = $2B6         ;INVERSE VIDEO FLAG (ATARI KEY)
	FILFLG       = $2B7         ;RIGHT FILL FLAG FOR DRAW
	TMPROW       = $2B8
	TMPCOL       = $2B9
	SCRFLG       = $2BB         ;SET IF SCROLL OCCURS
	HOLD4        = $2BC         ;MORE DRAW TEMPS
	HOLD5        = $2BD
	SHFLOK       = $2BE         ;SHIFT LOCK KEY
	BOTSCR       = $2BF         ;BOTTOM OF SCREEN (24 NORM, 4 SPLIT)

	; 4 bytes, The player 0 to 3 color shadow registers.
	PCOLR0       = $2C0
	PCOLR1       = $2C1
	PCOLR2       = $2C2
	PCOLR3       = $2C3

	; 4 bytes, The playfield 0 to 3 color shadow registers.
	COLOR0       = $2C4
	COLOR1       = $2C5
	COLOR2       = $2C6
	COLOR3       = $2C7

	; Byte, The background color shadow register.
	COLOR4       = $2C8

	; $2C9 - $2DF are unused

	GLBABS       = $2E0         ;GLOBAL VARIABLES

	; $2E0 - $2E3 are unused

	; Byte, The high byte of the amount of memory available. $A0 for 48K. $C0 if OS ROM enabled on a 64+K Atari.
	RAMSIZ       = $2E4

	MEMTOP       = $2E5         ;TOP OF AVAILABLE MEMORY
	MEMLO        = $2E7         ;BOTTOM OF AVAILABLE MEMORY

	; $2E9 is unused

	; 4 bytes, Device status. Only the first has a name, even though each has a unique purpose.
	DVSTAT       = $2EA         ;STATUS BUFFER

	CBAUDL       = $2EE         ;CASSETTE BAUD RATE (LO BYTE)
	CBAUDH       = $2EF         ;   "      "    "   (HI BYTE)
	CRSINH       = $2F0         ;CURSOR INHIBIT (00 = CURSOR ON)
	KEYDEL       = $2F1         ;KEY DELAY
	CH1          = $2F2

	; CHACTL shadow register.
	CHACT        = $2F3

	; CHBASE shadow register.
	CHBAS        = $2F4

	; $2F5 - $2F9 are unused

	CHAR         = $2FA

	; Byte, The last read/write byte or color used.
	ATACHR       = $2FB

	CH           = $2FC         ;GLOBAL VARIABLE FOR KEYBOARD
	FILDAT       = $2FD         ;RIGHT FILL DATA (DRAW)
	DSPFLG       = $2FE         ;DISPLAY FLAG: DISP CONTROLS IF NON-ZERO
	SSFLAG       = $2FF         ;START/STOP FLAG (CNTL-1) FOR PAGING


	; Page 3 ========================================================================================

	; 12 bytes, The device control block. This is used by the OS SIO.
	DCB          = $300

	; Byte, The device ID number. eg, '1' or $31 for "D1".
	DDEVIC       = $300

	; Byte, The unit number. This is just added to DDEVIC and not actually used in the hardware. This
	; allows specifying $31 for DDEVIC and 1 here for "D1", 2 here for "D2", etc.
	DUNIT        = $301

	; Byte, The command to use.
	DCOMND       = $302

	; Byte, The status returned by the command.
	DSTATS       = $303

	; Address, Memory pointer for disk load/save routines.
	DBUF         = $304
	DBUFLO       = $304
	DBUFHI       = $305

	; Byte, Device time out in 1s units.
	DTIMLO       = $306

	; Not used.
	DUNUSE       = $307

	; Word, The byte count to pass to SIO.
	DBYT         = $308
	DBYTLO       = $308
	DBYTHI       = $309

	; Word/2 bytes, Auxiliary bytes for SIO. Sector to use for disk load/save routines.
	DAUX         = $30A
	DAUX1        = $30A
	DAUX2        = $30B

	TIMER1       = $30C         ;INITIAL TIMER VALUE
	ADDCOR       = $30E         ;ADDITION CORRECTION
	CASFLG       = $30F         ;CASSETTE MODE WHEN SET
	TIMER2       = $310         ;FINAL TIME VALUE (USED TO COMPUTE BAUD RATE)
	TEMP1        = $312         ;TEMP LOCATIONS
	TEMP2        = $314         ; ...
	TEMP3        = $315         ; ...
	SAVIO        = $316         ;SAVE SERIAL IN DATA PORT
	TIMFLG       = $317         ;TIME OUT FLAG FOR BAUD RATE CORRECTION

	; Byte, SIO's saved stack pointer.
	STACKP       = $318

	TSTAT        = $319         ;TEMP STATUS LOC

	HATABS       = $31A         ;HANDLER ADDRESS TABLE

	; IOCB Base Addresses -----------------------------------------------------------------
	; 8x 16 bytes. IOCB OFFSETS for IOCB0. Each IOCB block is 16 bytes in length.

	; 16 bytes, General offset for the beginning of the IOCB data.
	IOCB         = $340

	; Byte, Handler index. $FF if free.
	ICHID        = $340

	; Byte, Device/drive number.
	ICDNO        = $341

	; Byte, Command code. See ATARIOS_ICCOM::*.
	ICCOM        = $342

	; Byte, Status. See ATARIOS_CIO_STATUS::*.
	ICSTA        = $343

	; Address, Buffer address.
	ICBA         = $344
	ICBAL        = $344
	ICBAH        = $345

	; Address, Put byte routine address - 1 (for using with an RTS).
	ICPT         = $346
	ICPTL        = $346
	ICPTH        = $347

	; Word, Buffer length.
	ICBL         = $348
	ICBLL        = $348
	ICBLH        = $349

	; 6 bytes, Auxiliary 1 to 6.
	; For Open commands, see ATARIOS_CIO_OPENAUX::*
	ICAX1        = $34A
	ICAX2        = $34B
	ICAX3        = $34C
	ICAX4        = $34D
	ICAX5        = $34E
	ICAX6        = $34F

	; 4 bytes, ???
	ICSPR        = $34C

	; End IOCB Base Addresses -------------------------------------------------------------

	; 40 bytes, The printer buffer.
	PRNBUF       = $3C0

  ; $3EA - $3FC are unused

	; 132 bytes ($3FD - $47F). Cassette buffer. The first four bytes are control bytes. The other 128
	; bytes are the actual data buffer.
	CASBUF       = $3FD


	; Page 4 ========================================================================================

	; Address, The start of available memory for user code.
	USAREA       = $480


	; Floating Point Library Vectors ================================================================

	; ATASCII to floating point.
	AFP          = $D800

	; Floating point to ATASCII
	FASC         = $D8E6

	; Integer to floating point
	IFP          = $D9AA

	; Floating point to Integer
	FPI          = $D9D2

	; FR0-FR1 Subtraction
	FSUB         = $DA60

	; FR0+FR1 Addition
	FADD         = $DA66

	; FR0*FR1 Multiplication
	FMUL         = $DADB

	; FR0/FR1 Division
	FDIV         = $DB28

	; Floating Load 0 using X,Y
	FLD0R        = $DD89

	; Floading Load 0 using FLPTR
	FLD0P        = $DD8D

	; Floating Load 1 using X,Y
	FLD1R        = $DD98

	; Floating Load 1 using FLPTR
	FLD1P        = $DD9C

	; Floating store using X,Y
	FSTOR        = $DDA7

	; Floating store with FLPTR
	FSTOP        = $DDA8

	; Move FR0
	FMOVE        = $DDB6

	; Polynomial evaluation
	PLYEVL       = $DD40

	; Exponentiation - e**FR0
	EXP          = $DDC0

	; Exponentiation - 10**FR0
	EXP1O        = $DDCC

	; Natural logarithm
	LOG          = $DECD

	; Base 10 logarithm
	LOG10        = $DED1

	; Set to zero
	ZFR0         = $DA44

	; Set register in X to zero
	AF1          = $DA46


	; Misc ==========================================================================================

	; Address, Where the OS font is located. 1KB in length.
	FONT         = $E000


	; Vector Tables =================================================================================
	; These are 8 addresses each, but finding a list of the actual routines they reference has proven
	; to be difficult. Push these addresses then RTS.

	; 8 addresses, Editor vector table.
	EDITRV       = $E400

	; Address, Puts a single character on the screen.
	EOUTCHV      = $E406

	; 8 addresses. Screen vector table.
	SCRENV       = $E410

	; 8 addresses. Keyboard vector table.
	KEYBDV       = $E420

	; Address, Get a character from the keyboard.
	KGETCHV      = $E424

	; 8 addresses. Printer vector table.
	PRINTV       = $E430

	; 8 addresses. Cassette vector table.
	CASETV       = $E440


	; Routine Vectors ===============================================================================
	; These occupy 3 bytes each because they are "JMP $abcd" vectors. Just JSR to them to access the
	; potentially changed routine addresses in the OS.

	; Disk initialize.
	DISKIV       = $E450

	; Disk interface.
	DSKINV       = $E453

	; Vector for CIO commands.
	CIOV         = $E456

	; Vector for SIO commands.
	SIOV         = $E459

	; Set vertical blank.
	SETVBV       = $E45C

	; System vertical blank.
	SYSVBV       = $E45F

	; Exit vertical blank.
	XITVBV       = $E462

	; SIO initialize.
	SIOINV       = $E465

	; Send enable.
	SENDEV       = $E468

	; Interrupt handler.
	INTINV       = $E46B

	; CIO initialize.
	CIOINV       = $E46E

	; Blackboard mode.
	BLKBDV       = $E471

	; Warm start entry.
	WARMSV       = $E474

	; Cold start entry.
	COLDSV       = $E477

	; Cassette: Read block.
	RBLOKV       = $E47D

	; Cassette: Open for input.
	DSOPIV       = $E480
.ENDSCOPE ; AtariOS



; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Constants

; *************************************************************************************************
; CIO values

; ICCOM Values ====================================================================================
.ENUM ATARIOS_ICCOM
	; Open a file.
	Open              = $03 ; OPEN

	; Get text data.
	GetText           = $05 ; GETREC

	; Get binary data.
	GetBinary         = $07 ; GETCHR

	; Put text data.
	PutText           = $09 ; PUTREC

	; Put binary data.
	PutBinary         = $0B ; PUTCHR

	; Close a file.
	Close             = $0C ; CLOSE

	; Request status.
	Status            = $0D ; STATIS

	; Special commands.
	Special           = $0E ; SPECIL

	; Draw line.
	DrawLine          = $11 ; DRAWLN

	; Draw a line, filling in to the right.
	DrawLineFill      = $12 ; FILLIN

	; Rename disk file.
	Rename            = $20 ; RENAME

	; Delete disk file.
	Delete            = $21 ; DELETE

	; Format disk.
	Format            = $22 ; FORMAT

	; Lock file.
	Lock              = $23 ; LOCKFL

	; Unlock file.
	Unlock            = $24 ; UNLOCK

	; Point sector.
	Point             = $25 ; POINT

	; Note sector.
	Note              = $26 ; NOTE

	; Concurrent I/O mode.
	Concurrent        = $28 ; CCIO

	; Free.
	Free              = $FF ; IOCFRE
.ENDENUM


; CIO Auxiliary Commands on Open ==================================================================
.ENUM ATARIOS_CIO_OPENAUX
	; Open for Append.
	Append            = $01 ; APPEND

	; Directory access.
	Directory         = $02 ; DIRECT

	; For input.
	Input             = $04 ; OPNIN

	; For output.
	Output            = $08 ; OPNOT

	; For input and output (Input | Output)
	InputOutput       = $0C ; OPNINO

	; Mixed mode.
	MixedMode         = $10 ; MXDMOD

	; Do not clear the screen.
	NoClear           = $20 ; INSCLR
.ENDENUM ; ATARIOS_CIO_OPENAUX


; CIO Error Codes ================================================================================
; Bit 7 is set on error.
.ENUM ATARIOS_CIO_STATUS
	; Successful.
	Success           = $01 ; SUCCES

	; Aborted by break.
	Break             = $80 ; BRKABT

	; IOCB already open.
	AlreadyOpen       = $81 ; PRVOPN

	; Non-existent device.
	NotExist          = $82 ; NONDEV

	; Write-only.
	WriteOnly         = $83 ; WRONLY

	; Invalid command.
	InvalidCommand    = $84 ; NVALID

	; Device or file not open.
	NotOpen           = $85 ; NOTOPN

	; Invalid IOCB number.
	InvalidNumber     = $86 ; BADIOC

	; Read-only.
	ReadOnly          = $87 ; RDONLY

	; End of file.
	EndOfFile         = $88 ; EOFERR

	; Truncated record.
	Truncated         = $89 ; TRNRCD

	; Timeout.
	Timeout           = $8A ; TIMOUT

	; Device did not ACK.
	DeviceNotACK      = $8B ; DNACK

	; Serial bus framing error.
	FrameError        = $8C ; FRMERR

	; Cursor out of range.
	CursorOutOfRange  = $8D ; CRSROR

	; Serial bus overrun.
	Overrun           = $8E ; OVRRUN

	; Serial bus checksum mismatch.
	ChecksumMismatch  = $8F ; CHKERR

	; Device error: operation incomplete.
	DeviceError       = $90 ; DERROR

	; Bad screen mode.
	BadScreenMode     = $91 ; BADMOD

	; Function not in handler.
	InvalidFunction   = $92 ; FNCNOT

	; Insufficient memory for screen mode.
	OutOfScreenMemory = $93 ; SCRMEM
.ENDENUM



; *************************************************************************************************
; Misc OS constants
; These are unused, but included here in case we need to put them somewhere.
;IOCBSZ  = 16 ; NUMBER OF BYTES / IOCB
;MAXIOC  = 8*IOCBSZ ; LENGTH OF IOCB AREA
;MAXDEV  = $21 ; MAXIMUM HANDLER ADDRESS INDEX



.ENDIF ; .IFNDEF IncludeFile_AtariOS