; Atari System Named Constants
; License: GPL-2, see LICENSE
; Date Created: 2021-08-19
; (c) 2021 to 2022 Mark Ormston <memso@memso.net>
.FEATURE string_escapes
.LINECONT +

.IFNDEF IncludeFile_AtariSystem
IncludeFile_AtariSystem = 1
.INCLUDE "6502.inc"

; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Atari Hardware

; *************************************************************************************************
; GTIA

; =================================================================================================
; Write struct. Uses the standard Atari constant names.
.STRUCT GTIA_W
	.ORG $D000
	HPOSP0  .BYTE     ; $D000
	HPOSP1  .BYTE     ; $D001
	HPOSP2  .BYTE     ; $D002
	HPOSP3  .BYTE     ; $D003
	HPOSM0  .BYTE     ; $D004
	HPOSM1  .BYTE     ; $D005
	HPOSM2  .BYTE     ; $D006
	HPOSM3  .BYTE     ; $D007
	SIZEP0  .BYTE     ; $D008
	SIZEP1  .BYTE     ; $D009
	SIZEP2  .BYTE     ; $D00A
	SIZEP3  .BYTE     ; $D00B
	SIZEM   .BYTE     ; $D00C
	GRAFP0  .BYTE     ; $D00D
	GRAFP1  .BYTE     ; $D00E
	GRAFP2  .BYTE     ; $D00F
	GRAFP3  .BYTE     ; $D010
	GRAFM   .BYTE     ; $D011
	COLPM0  .BYTE     ; $D012
	COLPM1  .BYTE     ; $D013
	COLPM2  .BYTE     ; $D014
	COLPM3  .BYTE     ; $D015
	COLPF0  .BYTE     ; $D016
	COLPF1  .BYTE     ; $D017
	COLPF2  .BYTE     ; $D018
	COLPF3  .BYTE     ; $D019
	COLBK   .BYTE     ; $D01A
	PRIOR   .BYTE     ; $D01B
	VDELAY  .BYTE     ; $D01C
	GRACTL  .BYTE     ; $D01D
	HITCLR  .BYTE     ; $D01E
	CONSOL  .BYTE     ; $D01F
.ENDSTRUCT ; GTIA_W


; =================================================================================================
; Read struct. Uses the standard Atari constant names.
.STRUCT GTIA_R
	.ORG $D000
	M0PF    .BYTE     ; $D000
	M1PF    .BYTE     ; $D001
	M2PF    .BYTE     ; $D002
	M3PF    .BYTE     ; $D003
	P0PF    .BYTE     ; $D004
	P1PF    .BYTE     ; $D005
	P2PF    .BYTE     ; $D006
	P3PF    .BYTE     ; $D007
	M0PL    .BYTE     ; $D008
	M1PL    .BYTE     ; $D009
	M2PL    .BYTE     ; $D00A
	M3PL    .BYTE     ; $D00B
	P0PL    .BYTE     ; $D00C
	P1PL    .BYTE     ; $D00D
	P2PL    .BYTE     ; $D00E
	P3PL    .BYTE     ; $D00F
	TRIG0   .BYTE     ; $D010
	TRIG1   .BYTE     ; $D011
	TRIG2   .BYTE     ; $D012
	TRIG3   .BYTE     ; $D013
	PAL     .BYTE     ; $D014
	Unused  .BYTE 10  ; $D015 - $D01E
	CONSOL  .BYTE     ; $D01F
.ENDSTRUCT ; GTIA_R


; =================================================================================================
; Constant Values

; Collision -----------------------------------------------------------------------------
; This same data is used by M*PF, P*PF, M*PL, and P*PL
; Players never collide with themselves (their matching bit is always clear)
.ENUM GTIA_COLLISION
	With0             = $01
	With1             = $02
	With2             = $04
	With3             = $08
.ENDENUM

; SIZEP* --------------------------------------------------------------------------------
.ENUM GTIA_SIZEP
	Normal            = $00
	Double            = $01
	Quad              = $03
.ENDENUM

; SIZEM ---------------------------------------------------------------------------------
.ENUM GTIA_SIZEM
	Missile0Normal    = $00
	Missile0Double    = $01
	Missile0Quad      = $03

	Missile1Normal    = $00
	Missile1Double    = $04
	Missile1Quad      = $0C

	Missile2Normal    = $00
	Missile2Double    = $10
	Missile2Quad      = $30

	Missile3Normal    = $00
	Missile3Double    = $40
	Missile3Quad      = $C0
.ENDENUM

; PRIOR ---------------------------------------------------------------------------------
.ENUM GTIA_PRIOR
	; Priority types
	; Pl = Player (Multiple means 0-1 for the first, 2-3 for the second)
	; PF = Playfields (Multiple means 0-1 for the first, 2-3 for the second)
	Pl_PF             = $01 ; P0 > P1 > P2 > P3 > PF0 > PF1 > PF2 > PF3 > BAK
	Pl_PF_Pl          = $02 ; P0 > P1 > PF0 > PF1 > PF2 > PF3 > P2 > P3 > BAK
	PF_Pl             = $04 ; PF0 > PF1 > PF2 > PF3 > P0 > P1 > P2 > P3 > BAK
	PF_Pl_PF          = $08 ; PF0 > PF1 > P0 > P1 > P2 > P3 > PF2 > PF3 > BAK
	PriorityMask      = $0F

	FifthPlayer       = $10
	MulticolorPlayer  = $20

	; GTIA modes
	NormalMode        = $00
	LumaMode          = $40
	ManyColorMode     = $80
	RainbowMode       = $C0
	ModeMask          = $C0
.ENDENUM

; VDELAY --------------------------------------------------------------------------------
.ENUM GTIA_VDELAY
	Missile0          = $01
	Missile1          = $02
	Missile2          = $04
	Missile3          = $08
	Player0           = $10
	Player1           = $20
	Player2           = $40
	Player3           = $80
.ENDENUM

; GRACTL --------------------------------------------------------------------------------
.ENUM GTIA_GRACTL
	MissileDMA        = $01
	PlayerDMA         = $02
	TriggerLatch      = $04
.ENDENUM

; CONSOL --------------------------------------------------------------------------------
.ENUM GTIA_CONSOL
	Start             = $01
	Select            = $02
	Option            = $04
	Loudspeaker       = $08
	; The value to write to CONSOL to then read the SSO state
	WriteSSOState     = $08
.ENDENUM



; *************************************************************************************************
; POKEY

; =================================================================================================
; Write struct. Uses the standard Atari constant names.
.STRUCT POKEY_W
	.ORG $D200
	AUDF1   .BYTE     ; $D200
	AUDC1   .BYTE     ; $D201
	AUDF2   .BYTE     ; $D202
	AUDC2   .BYTE     ; $D203
	AUDF3   .BYTE     ; $D204
	AUDC3   .BYTE     ; $D205
	AUDF4   .BYTE     ; $D206
	AUDC4   .BYTE     ; $D207
	AUDCTL  .BYTE     ; $D208
	STIMER  .BYTE     ; $D209
	SKREST  .BYTE     ; $D20A
	POTGO   .BYTE     ; $D20B
	Unused  .BYTE     ; $D20C
	SEROUT  .BYTE     ; $D20D
	IRQEN   .BYTE     ; $D20E
	SKCTL   .BYTE     ; $D20F
.ENDSTRUCT ; POKEY_W


; =================================================================================================
; Read struct. Uses the standard Atari constant names.
.STRUCT POKEY_R
	.ORG $D200
	POT0    .BYTE     ; $D200
	POT1    .BYTE     ; $D201
	POT2    .BYTE     ; $D202
	POT3    .BYTE     ; $D203
	POT4    .BYTE     ; $D204
	POT5    .BYTE     ; $D205
	POT6    .BYTE     ; $D206
	POT7    .BYTE     ; $D207
	ALLPOT  .BYTE     ; $D208
	KBCODE  .BYTE     ; $D209
	RANDOM  .BYTE     ; $D20A
	Unused  .BYTE 2   ; $D20B-$D20C
	SERIN   .BYTE     ; $D20D
	IRQST   .BYTE     ; $D20E
	SKSTAT  .BYTE     ; $D20F
.ENDSTRUCT ; POKEY_R


; =================================================================================================
; Constant values

; IRQ Flags -----------------------------------------------------------------------------
.ENUM POKEY_IRQ
	; Setting flags
	Timer1            = $01
	Timer2            = $02
	Timer4            = $04
	Transmission      = $08
	SerialOutput      = $10
	SerialInput       = $20
	Keyboard          = $40
	BreakKey          = $80

	; Clearing status flags (Transmission doesn't support this)
	ClearTimer1       = $FE
	ClearTimer2       = $FD
	ClearTimer4       = $FB
	ClearSerialOutput = $EF
	ClearSerialInput  = $DF
	ClearKeyboard     = $BF
	ClearBreakKey     = $7F
.ENDENUM

; AUDC* ---------------------------------------------------------------------------------
.ENUM POKEY_AUDC
	; Only one of:
	Mute              = $00
	Volume1           = $01
	Volume2           = $02
	Volume3           = $03
	Volume4           = $04
	Volume5           = $05
	Volume6           = $06
	Volume7           = $07
	Volume8           = $08
	Volume9           = $09
	VolumeA           = $0A
	VolumeB           = $0B
	VolumeC           = $0C
	VolumeD           = $0D
	VolumeE           = $0E
	VolumeF           = $0F
	VolumeMask        = $0F

	VolumeOnly        = $10
	PureTone          = $20
	Polynomial4Bit    = $40
	TimerAsClock      = $80
.ENDENUM


; AUDCTL --------------------------------------------------------------------------------
.ENUM POKEY_AUDCTL
	Use15KHzSlow      = $01
	Channel2HighPass  = $02
	Channel1HighPass  = $04
	Link3And4         = $08
	Link1And2         = $10
	Channel3Fast      = $20
	Channel1Fast      = $40
	Polynomial9Bit    = $80
.ENDENUM

; KBCODE --------------------------------------------------------------------------------
.ENUM POKEY_KBCODE
	; Mask just for what key is pressed
	KeyMask           = $3F
	Shift             = $40
	Control           = $80
.ENDENUM

; SKSTAT --------------------------------------------------------------------------------
.ENUM POKEY_SKSTAT
	SerialInput       = $02
	Keypress          = $04
	Shift             = $08
	InputDataLine     = $10
	InputOverrun      = $20
	KeyboardOverrun   = $40
	InputFrameError   = $80
.ENDENUM

; SKCTL ---------------------------------------------------------------------------------
.ENUM POKEY_SKCTL
	KeyboardDebounce  = $01
	KeyboardScan      = $02
	FastPotScan       = $04
	TwoToneMode       = $08
	AsyncReceive      = $10
	ClockOutInternal  = $20
	ClockInInternal   = $40
	ForceBreak        = $80
.ENDENUM

; SIO Commands --------------------------------------------------------------------------
.ENUM POKEY_SIOCOMMAND
	Download          = $20 ; ' ' - 810 and 815 only
	Format            = $21 ; '!'
	FormatED          = $22 ; '"'
	GetSpeed          = $3F ; '?' - Most high-speed drives
	HappyControl      = $48 ; 'H' - Special control codes for Happy 810 and 1050 drives
	ReadPercom        = $4E ; 'N' - Double-density capable drives
	WritePercom       = $4F ; 'O' - Double-density capable drives
	Put               = $50 ; 'P' - Write without verification
	ReadSpin          = $51 ; 'Q'
	Read              = $52 ; 'R'
	Status            = $53 ; 'S'
	ReadAddress       = $54 ; 'T'
	MotoOnN           = $55 ; 'U'
	VerifySector      = $56 ; 'V' - 815-only
	Write             = $57 ; 'W' - Write with verification
.ENDENUM

; SIO Devices ---------------------------------------------------------------------------
.ENUM POKEY_SIODEVICEID
	; Disk drives (D:)
	DiskMin           = $31
	DiskMax           = $3F

	; Printers (P:)
	PrinterMin        = $40
	PrinterMax        = $43

	; 850 Serial Device (R:)
	Serial850Min      = $50
	Serial850Max      = $53

	; 1030 Serial Device (T:)
	Serial1030        = $58

	; Cassette (C:) (used by AtariOS only)
	Cassette          = $5F

	; Modern devices
	APE               = $45 ; (Atari Peripheral Emulator)
	AspeQt            = $46
	DOS2DOS           = $6F ; (PCLink)
	SIO2USB           = $70
	SIO2SD            = $72
.ENDENUM



; *************************************************************************************************
; PIA

; =================================================================================================
; Memory Struct
.STRUCT PIA
	.ORG $D300
	PORTA   .BYTE     ; $D300
	PORTB   .BYTE     ; $D301
	PACTL   .BYTE     ; $D302
	PBCTL   .BYTE     ; $D303
.ENDSTRUCT ; PIA


; =================================================================================================
; Constant Values

; XLPORTB -------------------------------------------------------------------------------
.ENUM PIA_XLPORTB
	; PortB value on XL and higher
	KernelEnabled     = $01
	BASICDisabled     = $02
	SelfTestDisabled  = $80

	; 1200XL only:
	LED1Disabled      = $04
	LED2Disabled      = $08

	; XE Only:
	Bank0             = $00
	Bank1             = $04
	Bank2             = $08
	Bank3             = $0C
	Bank_Mask         = $0C
	BankDisabled      = $10
	BankAccessANTIC   = $20
.ENDENUM

; CTL ---------------------------------------------------------------------------------
; These are used for PACTL and PBCTL
.ENUM PIA_CTL
	IRQ1Enabled                 = $01
	IRQ1Positive                = $02
	AccessIORegisters           = $04

	; SIO flag values, only use one. 
	SIO_Mask                    = $38
	SIO_Input_Negative_Disabled = $00
	SIO_Input_Negative_Enabled  = $08
	SIO_Input_Positive_Disabled = $10
	SIO_Input_Positive_Enabled  = $18
	SIO_Output_LowerOnWrite     = $20
	SIO_Output_PulseLowOnWrote  = $28
	SIO_Output_AssertLine       = $30
	SIO_Output_NegateLine       = $38

	; Read Only:
	IRQ2Pending                 = $40
	IRQ1Pending                 = $80
.ENDENUM


; =================================================================================================
; XLPORTB / Extended RAM Values

; Address, The beginning of the RAM swapped out by extended memory.
PIA_XLPORTB_MemoryBank_Start = $4000
PIA_XLPORTB_MemoryBank_End   = $7FFF

; Word, Size of the XLPORTB memory bank.
PIA_XLPORTB_MEMORYBANK_SIZE  = PIA_XLPORTB_MemoryBank_End - PIA_XLPORTB_MemoryBank_Start + 1



; *************************************************************************************************
; ANTIC

; =================================================================================================
; Write struct. Uses the standard Atari constant names.
.STRUCT ANTIC_W
	.ORG $D400
	DMACTL  .BYTE     ; $D400
	CHACTL  .BYTE     ; $D401
	DLISTL  .BYTE     ; $D402
	DLISTH  .BYTE     ; $D403
	HSCROL  .BYTE     ; $D404
	VSCROL  .BYTE     ; $D405
	Unused0 .BYTE     ; $D406
	PMBASE  .BYTE     ; $D407
	Unused1 .BYTE     ; $D408
	CHBASE  .BYTE     ; $D409
	WSYNC   .BYTE     ; $D40A
	Unused2 .BYTE 3   ; $D40B - $D40D
	NMIEN   .BYTE     ; $D40E
	NMIRES  .BYTE     ; $D40F
.ENDSTRUCT ; ANTIC_W


; =================================================================================================
; Read struct. Uses the standard Atari constant names.
.STRUCT ANTIC_R
	.ORG $D400
	Unused0 .BYTE 11  ; $D400 - $D40A
	VCOUNT  .BYTE     ; $D40B
	PENH    .BYTE     ; $D40C
	PENV    .BYTE     ; $D40D
	Unused1 .BYTE     ; $D40E
	NMIST   .BYTE     ; $D40F
.ENDSTRUCT ; ANTIC_R


; =================================================================================================
; Constant values

; DMACTL --------------------------------------------------------------------------------
.ENUM ANTIC_DMACTL
	; Playfield types. Only use one
	Playfield_None    = $00
	Playfield_Narrow  = $01
	Playfield_Normal  = $02
	Playfield_Wide    = $03
	Playfield_Mask    = $03

	; Other values
	MissileDMA        = $04
	PlayerDMA         = $08
	PlayerResolution  = $10
	DisplayListDMA    = $20
.ENDENUM

; CHACTL --------------------------------------------------------------------------------
.ENUM ANTIC_CHACTL
	BlinkHide         = $01
	InvertEnable      = $02
	ReflectEnable     = $04
.ENDENUM

; NMI values ----------------------------------------------------------------------------
.ENUM ANTIC_NMI
	SystemReset       = $20 ; Atari 400/800 only, and only in NMIST
	VerticalBlank     = $40
	DisplayList       = $80
.ENDENUM


; =================================================================================================
; Display List Commands
.ENUM ANTIC_DISPLAYLIST
	; Blanks ------------------------------------------------------------------------------
	Blank1            = $00
	Blank2            = $10
	Blank3            = $20
	Blank4            = $30
	Blank5            = $40
	Blank6            = $50
	Blank7            = $60
	Blank8            = $70
	Blank1DLI         = $80
	Blank2DLI         = $90
	Blank3DLI         = $A0
	Blank4DLI         = $B0
	Blank5DLI         = $C0
	Blank6DLI         = $D0
	Blank7DLI         = $E0
	Blank8DLI         = $F0

	; Address changes ---------------------------------------------------------------------
	SwitchAddress     = $01
	SwitchAddressDLI  = $81
	NextAddress       = $41

	; By Mode -----------------------------------------------------------------------------
	; Smooth scrolling values not included

	; Mode 2 ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
	Mode2             = $02
	Mode2SetAddr      = $42
	Mode2DLI          = $82
	Mode2SetAddrDLI   = $C2

	; Mode 3 ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
	Mode3             = $03
	Mode3SetAddr      = $43
	Mode3DLI          = $83
	Mode3SetAddrDLI   = $C3

	; Mode 4 ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
	Mode4             = $04
	Mode4SetAddr      = $44
	Mode4DLI          = $84
	Mode4SetAddrDLI   = $C4

	; Mode 5 ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
	Mode5             = $05
	Mode5SetAddr      = $45
	Mode5DLI          = $85
	Mode5SetAddrDLI   = $C5

	; Mode 6 ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
	Mode6             = $06
	Mode6SetAddr      = $46
	Mode6DLI          = $86
	Mode6SetAddrDLI   = $C6

	; Mode 7 ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
	Mode7             = $07
	Mode7SetAddr      = $47
	Mode7DLI          = $87
	Mode7SetAddrDLI   = $C7

	; Mode 8 ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
	Mode8             = $08
	Mode8SetAddr      = $48
	Mode8DLI          = $88
	Mode8SetAddrDLI   = $C8

	; Mode 9 ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
	Mode9             = $09
	Mode9SetAddr      = $49
	Mode9DLI          = $89
	Mode9SetAddrDLI   = $C9

	; Mode A ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
	ModeA             = $0A
	ModeASetAddr      = $4A
	ModeADLI          = $8A
	ModeASetAddrDLI   = $CA

	; Mode B ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
	ModeB             = $0B
	ModeBSetAddr      = $4B
	ModeBDLI          = $8B
	ModeBSetAddrDLI   = $CB

	; Mode C ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
	ModeC             = $0C
	ModeCSetAddr      = $4C
	ModeCDLI          = $8C
	ModeCSetAddrDLI   = $CC

	; Mode D ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
	ModeD             = $0D
	ModeDSetAddr      = $4D
	ModeDDLI          = $8D
	ModeDSetAddrDLI   = $CD

	; Mode E ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
	ModeE             = $0E
	ModeESetAddr      = $4E
	ModeEDLI          = $8E
	ModeESetAddrDLI   = $CE

	; Mode F ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
	ModeF             = $0F
	ModeFSetAddr      = $4F
	ModeFDLI          = $8F
	ModeFSetAddrDLI   = $CF
.ENDENUM



; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Atari Constants

; *************************************************************************************************
; Keyboard Values

; =================================================================================================
; Atari keyboard scan codes (6 bits, so $00 to $3F)
.ENUM ATARI_KEYCODE
	CharA             = $3F
	CharB             = $15
	CharC             = $12
	CharD             = $3A
	CharE             = $2A
	CharF             = $38
	CharG             = $3D
	CharH             = $39
	CharI             = $0D
	CharJ             = $01
	CharK             = $05
	CharL             = $00
	CharM             = $25
	CharN             = $23
	CharO             = $08
	CharP             = $0A
	CharQ             = $2F
	CharR             = $28
	CharS             = $3E
	CharT             = $2D
	CharU             = $0B
	CharV             = $10
	CharW             = $2E
	CharX             = $16
	CharY             = $2B
	CharZ             = $17

	Num1              = $1F
	Num2              = $1E
	Num3              = $1A
	Num4              = $18
	Num5              = $1D
	Num6              = $1B
	Num7              = $33
	Num8              = $35
	Num9              = $30
	Num0              = $32

	Space             = $21
	Comma             = $20
	Period            = $22
	Semicolon         = $02
	Asterisk          = $07
	LessThan          = $36
	GreaterThan       = $37
	Plus              = $06
	Minus             = $0E
	Equal             = $0F
	Slash             = $26

	Return            = $0C
	Tab               = $2C
	Delete            = $34
	CapsLock          = $3C
	Invert            = $27
	Escape            = $1C
	Help              = $11

	; XLM only:
	F1                = $03
	F2                = $04
	F3                = $13
	F4                = $14

	; Not used: $09, $19, $24, $29, $31, $3B

	; These are NOT real scan codes, but are used when acting like they are real keys:
	Start             = $09
	Select            = $19
	Option            = $29
	Break             = $31
.ENDENUM


; =================================================================================================
; Special ATASCII Values
.ENUM ATASCII
	Delete            = $08
	Newline           = $0A
	Return            = $0D
	UpArrow           = $1C ; .ATURW
	DownArrow         = $1D ; .ATDRW
	LeftArrow         = $1E ; .ATLRW
	RightArrow        = $1F ; .ATRRW
	ClearScreen       = $7D ; .ATCLR
	Backspace         = $7E ; .ATRUB / RUBOUT
	Tab               = $7F ; .ATTAB
	EndOfLine         = $9B ; .ATEOL
	Bell              = $FD ; .ATBEL
.ENDENUM



; *************************************************************************************************
; Misc

; Byte, The size of a sector. Not necessarily true for double density disks, but they can be handled
; separately.
SECTORSIZE          = 128



; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Atari System Macros

; *************************************************************************************************
; Converts an ASCII/ATASCII character to an ANTIC character.
; Incoming:
;   character       Byte, A single character to convert to ANTIC.
; Size: 1
.MACRO AtasciiToANTIC character
  .IF (character<$20)
    .BYTE (character+$40)
  .ELSEIF (character<$60)
    .BYTE (character-$20)
  .ELSEIF (character<$80)
    .BYTE character
  .ELSEIF (character<$A0)
    .BYTE (character+$40)
  .ELSEIF (character<$E0)
    .BYTE character-$20
  .ELSE
    .BYTE character
  .ENDIF
.ENDMACRO



; *************************************************************************************************
; Converts ASCII/ATASCII characters to ANTIC characters.
; Incoming:
;   string          Address, The character string to convert to ANTIC characters.
; Size: Length of the string.
.MACRO StrAtasciiToANTIC string
  .REPEAT .STRLEN(string), I
    .SCOPE
      Char = .STRAT(string, I)
      AtasciiToANTIC Char
    .ENDSCOPE
  .ENDREP
.ENDMACRO

.ENDIF ; .IFNDEF IncludeFile_AtariSystem
