; Definitions for highspeed SIO code
; License: GPL-2, see LICENSE
; This Version (c) 2022 Mark Ormston <memso@memso.net>
; Original copyright (c) 2003-2010 by Matthias Reichl <hias@horus.com>

.IFNDEF IncludeFile_HISIO
IncludeFile_HISIO = 1
.INCLUDE "AtariOS.inc"

; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Addresses

; *************************************************************************************************
; Zero Page

; AtariOS::ZP_BFENLO/HI ($34/$35) are used as a data length instead of a pointer to the end of the
; buffer.

; $36/$37 are kept free to avoid troubles with TurboDOS

; CUSTOM SIO ----------------------------------------------------------------------------
; Matthias: Re-using original SIO ZP locations $38 - $3C for my own code

; Byte, Command frame retries.
HISIO_CRETRY = $38

; Byte, Data frame retries.
HISIO_DRETRY = $39

; Byte, Speed used by this SIO command (changed to standard for second retry). This is passed to
; POKEY_W::AUDF3.
HISIO_SPEED = $3A

; Word, Timer in ZP, used by fast VBI code.
HISIO_MYTIM1 = $3B

; /CUSTOM SIO ---------------------------------------------------------------------------



; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Constants

; Default number of command frame retries and the low byte of the time out
.IFDEF BT
	DEFCRETRY = $05
	T2TIMEOUT = $10
.ELSE
	DEFCRETRY = $0E
	T2TIMEOUT = $02
.ENDIF

; Number of command frame retries when checking highspeed mode
DETCRETRY = $05

; Pokey POKEY_W::SKCTL value for transmitting data (sync. send)
SKSEND = POKEY_SKCTL::KeyboardDebounce | POKEY_SKCTL::KeyboardScan | POKEY_SKCTL::ClockOutInternal

; Pokey POKEY_W::SKCTL value for receiving data (async. recv)
SKRECV = POKEY_SKCTL::KeyboardDebounce | POKEY_SKCTL::KeyboardScan | POKEY_SKCTL::AsyncReceive


; =================================================================================================
; Specific speed values

; Byte, The standard baud rate speed divisor.
STDSPD = $28

; Byte, The minimum special flag value for the speed (bit 6 set).
MINSPECIALSPEED = $40

.IFNDEF USONLY
	; Speed divisor rate for the 1050 Turbo.
	TURSPD = 6

	; Speed divisor rate for the XF551 / Happy Warp.
	XFSPD = 16

	; Byte, Values to use in the speed value to mark the 1050 Turbo, XF551, and Happy Warp drives.
	; TURFLG sets bit 7. XFFLG and HAPFLG set bit 6.
	; Remember, the slowest speed is $28 which will never have those bits set.
	TURFLG = $80
	XFFLG = $40
	HAPFLG = $41
.ENDIF


; =================================================================================================
; POKEY_W::IRQEN masks

; standard masks: break + send/receive
MSKRECV = POKEY_IRQ::BreakKey | POKEY_IRQ::SerialInput
MSKSEND = POKEY_IRQ::BreakKey | POKEY_IRQ::SerialOutput

.ENDIF
