; Support macros for debugging using an external display.
; License: GPL-2, see LICENSE
; This Version (c) 2022 Mark Ormston <memso@memso.net>
; Original copyright (c) 2005 Matthias Reichl

.IFNDEF IncludeFile_HWDebug
IncludeFile_HWDebug = 1

; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Addresses

; Address, Where to output display data.
HWDISP	            = $D5F0



; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Macros

; *************************************************************************************************
; Clears the debug display.
.MACRO DBGCLR
	.IFNDEF(HWDEBUG)
		.EXITMACRO
	.ENDIF

	JSR SDBGCLR
.ENDMACRO



; *************************************************************************************************
; Outputs an 8-character string.
; Parameters:
;   string          8 bytes, The string to copy to HWDISP.
.MACRO DBGTXT string
	.IFNDEF(HWDEBUG)
		.EXITMACRO
	.ENDIF

	.LOCAL DBGTXTT, DBGTXT1
	LDX #0
	BEQ DBGTXT1
DBGTXTT:
	.BYTE string
DBGTXT1:
	LDA DBGTXTT,X
	STA HWDISP,X
	INX
	CPX #8
	BNE DBGTXT1
.ENDMACRO



; *************************************************************************************************
; Outputs a 4-character string to a specific position.
; Parameters:
;   pos             Byte, The offset into HWDISP to use.
;   string          4 bytes, The string to display.
.MACRO DBGT4 pos, string
	.IFNDEF(HWDEBUG)
		.EXITMACRO
	.ENDIF

	.LOCAL DBGTXTT, DBGTXT1
	LDX #pos
	LDY #0
	BEQ DBGTXT1
DBGTXTT:
	.BYTE string
DBGTXT1:
	LDA DBGTXTT,Y
	STA HWDISP,X
	INX
	INY
	CPY #4
	BNE DBGTXT1
.ENDMACRO



; *************************************************************************************************
; Outputs a 16-bit hex value at an address.
; Parameters:
;   offset          Byte, The offset.
;   addr            Address, The address of the 16-bit value to convert to hex.
.MACRO DBGADR offset, addr
	.IFNDEF(HWDEBUG)
		.EXITMACRO
	.ENDIF

  LDX #offset
	LDA addr + 1
	JSR SDBGHEX
	LDA addr
	JSR SDBGHEX
.ENDMACRO



; *************************************************************************************************
; Output A as 2 hex characters to position X.
.MACRO DBGA
	.IFNDEF(HWDEBUG)
		.EXITMACRO
	.ENDIF

	JSR SDBGHEX
.ENDMACRO



; *************************************************************************************************
; Output A to specified position.
; Parameters:
;   pos             Byte, The offset into HWDISP to use.
.MACRO DBGASC pos
	.IFNDEF(HWDEBUG)
		.EXITMACRO
	.ENDIF

	STA HWDISP + pos
.ENDMACRO

.ENDIF