; 6502 high speed SIO code OS patch program
; License: GPL-2, see LICENSE
; This Version (c) 2022 Mark Ormston <memso@memso.net>
; Original copyright (c) by Matthias Reichl <hias@horus.com>

.IFNDEF IncludeFile_Patch
IncludeFile_Patch = 1
.INCLUDE "HISIO.inc"

; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Constants

; indicate we are building the OS ROM PATCH
HIPATCH = 1

.IFDEF BT
	; SPEEDTB size, table for device speeds has only 4-bytes (when BT defined)!
	SPEEDTBSIZE = 4

	; HS Index for 57.600 Baud
	BT576K = $08
.ENDIF



; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Addresses

; *************************************************************************************************
; Patch addresses

; starting address of code patch
PDEST = $CC00

; original SIO code + JMP back
PSTD = $CC10

; powerup-code, checking for SHIFT (=cold boot)
PPWRUP = $CC18

; start of new highspeed code
PSIO = $CC30

; power-up code
; this contains "LDA $033D"
; See Main/PatchCode.asm
PUPCODE = $C2B3

; new NMI handler
;PNMI = $CF68

.IFDEF PATCHKEY
	; new keyboard IRQ handler
	PKEY = $CFB7

	; this contains "LDA $D209"
	KEYIRQ = $FC20
.ENDIF

; direct entry to hisio code
; A needs to be set to highspeed mode/divisor
PHISIO = $CFED

; 16 bytes version string
PVER = $CFF0



; *************************************************************************************************
; Various RAM Buffer Addresses

.IFDEF ROMABLE
	; leave the first 8 bytes at page 1 free, some programs use them
  RAMBUF = $0108
.ELSE
  RAMBUF = PDEST
.ENDIF

; speed byte for current operation
SIOSPEED = RAMBUF

; 8-byte table for drive speeds
SPEEDTB = RAMBUF + 1

.IFDEF BT
	; 4-byte table for last accessed devices
	DEVICETB = RAMBUF + 5
.ENDIF

.IFDEF PATCHKEY
  ; add a config byte to enable/disable highspeed SIO with keyboard
  HIENAB = RAMBUF + 9
  ; 4 bytes for get status
  HIBUF = RAMBUF + 10
.ELSE
  ; 4 bytes for get status
  HIBUF = RAMBUF + 9
.ENDIF

.IFDEF BT
	NEXTSPEEDSLOT = HIBUF + 4
.ENDIF

; address of reset handler
.IFDEF ROMABLE
	.IFDEF BT
		RESETAD = NEXTSPEEDSLOT + 1
	.ELSE
		; after end of highspeed SIO data
		RESETAD = HIBUF + 4
	.ENDIF
.ELSE
	; (almost) at beginning of page 1
	RESETAD = $0108
.ENDIF

.ENDIF
