; Detect highspeed drives
; License: GPL-2, see LICENSE
; This Version (c) 2022 Mark Ormston <memso@memso.net>
; Original copyright (c) by Matthias Reichl <hias@horus.com>

.INCLUDE "../Include/HISIO.inc"

; *************************************************************************************************
; This is called when the original OS SIO code would have been called.
HISIO:
	; The highspeed SIO patch checks for valid AtariOS::DUNIT values before, so skip this test to save some
	; memory
	.IFNDEF HIPATCH
		LDA AtariOS::DUNIT
		BNE L_DOSIO
	
		LDX #7		; AtariOS::DUNIT = 0 clears speed table
	L_CLRLP:
	L_ABS01:
		STA SPEEDTB,X
		DEX
		BPL L_CLRLP
		RTS
	.ENDIF

	; Fall through ------>
; End HISIO



; *************************************************************************************************
; ???
L_DOSIO:
	LDX AtariOS::DUNIT
L_ABS02:
	LDA SPEEDTB - 1,X
	BEQ L_DOCHK

	; bytes in SIO table are now speed + 1 to prevent endless loop in case device sends speed byte 0
	SEC
	SBC #1

	; Fall through ------>
; End L_DOSIO



; *************************************************************************************************
; ???
DOHIENTRY:
L_ABS03:
	STA SIOSPEED
L_ABS04:
	JMP DOHISIO
; End DOHIENTRY



; *************************************************************************************************
; ???
L_DOCHK:
	LDA AtariOS::ZP_SOUNDR
	PHA
	LDA #0
	STA AtariOS::ZP_SOUNDR

	LDA #STDSPD
L_ABS05:
	STA SIOSPEED
	LDY #$09
L_SAVCMD:
	LDA AtariOS::DCOMND,Y
	PHA 
L_ABS06:
	; Happy/Speedy Ultra SIO mode?
	LDA L_CMD3F,Y
	STA AtariOS::DCOMND,Y
	DEY 
	BPL L_SAVCMD

L_ABS07:
	JSR L_DODET

	; for testing happy 810 mode, change BMI into JMP and connect a happy 1050
	BMI L_NOULTRA

L_ABS08:
	LDA HIBUF
	CMP #$0A	; speed byte $0A is most certainly a Happy 1050
	BNE L_SETSP2

	; enable fast write for Happy 1050 (but not for other drives),
	; otherwise data might be corrupted
	; stupid, stupid Happy...
L_ABS09:
	JSR L_CHKHAP

L_ABS10:
	LDA HIBUF
L_ABS11:
	JMP L_SETSP2

L_NOULTRA:
	.IFNDEF USONLY
	L_ABS12A:
		JSR L_GETST2
		; bail out early if drive is not responding
		BMI L_CHKNOHI
	
		; Turbo 1050?
		LDA #TURFLG
	L_ABS12:
		JSR L_GETST
		BPL L_SETSPD
	
		; XF551?
		LDA #XFFLG
	L_ABS13:
		JSR L_GETST
		BPL L_SETSPD
	
		; Happy 810?
		LDA #HAPFLG
	L_ABS14:
		STA SIOSPEED
	L_ABS15:
		JSR L_CHKHAP
		BPL L_SETSPD
	.ENDIF

L_CHKNOHI:
	; no, just use standard SIO speed
	LDA #STDSPD
	BNE L_SETSP2



.IFNDEF USONLY
	; ***********************************************************************************************
	; ???
	L_GETST:
	L_ABS16:
		STA SIOSPEED
		; Fall through ------>
	; End L_GETST
	
	
	
	; ***********************************************************************************************
	; ???
	L_GETST2:
		LDA #4
		STA AtariOS::DBYT
		LDX #$53
		LDA #$40
		; Fall through ------>
	; End L_GETST2
.ENDIF


; *************************************************************************************************
; ???
L_DOIMM:
	STX AtariOS::DCOMND
	STA AtariOS::DSTATS
	; Fall through ------>
; End L_DOIMM



; *************************************************************************************************
; ???
L_DODET:
	LDA #1
	LDX #DETCRETRY
L_ABS17:
	JMP DOHIDET
; End L_DODET



; *************************************************************************************************
; Sends a Happy command. The data is in AUX1 and AUX2.
L_CHKHAP:
	; check for happy 810
	LDX #$48
	LDA #0
	BEQ L_DOIMM
; End L_CHKHAP



.IFNDEF USONLY
	; ***********************************************************************************************
	; Loads SIOSPEED to save the speed
	L_SETSPD:
	L_ABS18:
		LDA SIOSPEED
		; Fall through ------>
	; End L_SETSPD
.ENDIF



; *************************************************************************************************
; Save the speed and move to the next drive
L_SETSP2:
	LDX AtariOS::DUNIT
	; add 1 to value to prevent endless loop if device sends a speed byte of 0
	CLC
	ADC #1
L_ABS19:
	STA SPEEDTB - 1,X

L_RESCMD:
	LDY #$00
L_RESLP:
	PLA 
	STA AtariOS::DCOMND,Y
	INY 
	CPY #$0A
	BCC L_RESLP
	PLA
	STA AtariOS::ZP_SOUNDR
L_ABS20:
	JMP L_DOSIO
; End L_SETSP2



; *************************************************************************************************
; 10 bytes, This data is stored in AtariOS::DCOMND to detect Happy/Speedy Ultra SIO mode.
L_CMD3F:
	.BYTE $3F,$40
L_ABS21:
	.WORD HIBUF,2,1,$20

; This sets the following values:
;DCOMND = $3F - Get Speed
;DSTATS = $40 - Read
;DBUF = HIBUF
;DTIMLO = 2
;DUNUSE = 0
;DBYT = 1
;DAUX1 = $20 - Set flags bit 5 (Enable/Disable Write Buffering)
;DAUX2 = $0  - Enable Write Buffering
