; Check for SIO2BT and highspeed drives
; License: GPL-2, see LICENSE
; This version (c) 2022 Mark Ormston <memso@memso.net>
; Original Copyright (c) by Matthias Reichl <hias@horus.com>
;          Copyright (c) 2019-2020 by Marcin Sochacki <marcin.sochacki@gmx.net>

.INCLUDE "../Include/HISIO.inc"

; *************************************************************************************************
; ???
HISIO:
	; Set AtariOS::CDEVIC to AtariOS::DDEVIC + AtariOS::DUNIT - 1
	CLC
	LDA AtariOS::DDEVIC
	ADC AtariOS::DUNIT
	ADC #<(-1)
	STA AtariOS::CDEVIC

	; Fall through ------>
; End HISIO



; *************************************************************************************************
; ???
L_DOSIO:
	JSR L_GETHSINDEX
	BEQ L_DOCHK

	; bytes in SIO table are now speed + 1 to prevent endless loop in case device sends speed byte 0
	SEC
	SBC #1

	; Fall through ------>
; End L_DOSIO



; *************************************************************************************************
; ???
DOHIENTRY:
L_ABS03:
	STA SIOSPEED
L_ABS04:
	JMP DOHISIO
; End DOHIENTRY



; *************************************************************************************************
; ???
L_DOCHK:
L_ABS05:
	LDY #$09
L_SAVCMD:
	LDA AtariOS::DCOMND,Y
	PHA
L_ABS06:
	; Happy/Speedy Ultra SIO mode?
	LDA L_CMD3F,Y
	STA AtariOS::DCOMND,Y
	DEY
	BPL L_SAVCMD

	; test divisor $08 (SIO2BT 57.6Kbps)
	LDA #BT576K
	JSR L_GETST
	BPL L_SETSPD
	CPY #ATARIOS_CIO_STATUS::DeviceNotACK
	BEQ L_SETSPD

	LDA #STDSPD
	JSR L_GETST
	BPL L_TESTHISIO
	CPY #ATARIOS_CIO_STATUS::DeviceNotACK
	BEQ L_TESTHISIO
	; don't set speed if device timed out
	LDA #$FF
	STA SIOSPEED
	BNE L_RESCMD

L_TESTHISIO:
	; reinstate the speed poll command since get status wiped it out
	LDY #$09
L_SAVCMD2:
	; Happy/Speedy Ultra SIO mode?
	LDA L_CMD3F,Y
	STA AtariOS::DCOMND,Y
	DEY
	BPL L_SAVCMD2
L_ABS07:
	JSR L_DODET

	; for testing happy 810 mode, change BMI into JMP and connect a happy 1050
	BMI L_NOULTRA

L_ABS08:
	LDA HIBUF
	; speed byte $0A is most certainly a Happy 1050
	CMP #$0A
	BNE L_SETSP2

	; enable fast write for Happy 1050 (but not for other drives),
	; otherwise data might be corrupted
	; stupid, stupid Happy...
L_ABS09:
	JSR L_CHKHAP

L_ABS10:
	LDA HIBUF
L_ABS11:
	JMP L_SETSP2
; End L_DOCHK



; *************************************************************************************************
; ???
L_NOULTRA:
	.IFNDEF USONLY
	L_ABS12A:
		JSR L_GETST2
		; bail out early if drive is not responding
		BMI L_CHKNOHI

		; Turbo 1050?
		LDA #TURFLG
	L_ABS12:
		JSR L_GETST
		BPL L_SETSPD

		; XF551?
		LDA #XFFLG
	L_ABS13:
		JSR L_GETST
		BPL L_SETSPD

		; Happy 810?
		LDA #HAPFLG
	L_ABS14:
		STA SIOSPEED
	L_ABS15:
		JSR L_CHKHAP
		BPL L_SETSPD
	.ENDIF

L_CHKNOHI:
	; no, just use standard SIO speed
	LDA #STDSPD
	BNE L_SETSP2

	.IFNDEF USONLY
	L_GETST:
	L_ABS16:
		STA SIOSPEED

	L_GETST2:
		LDA #4
		STA AtariOS::DBYT
		LDX #$53
		LDA #$40
	.ENDIF

	; Fall through ------>
; End L_NOULTRA:



; *************************************************************************************************
; ???
L_DOIMM:
	STX AtariOS::DCOMND
	STA AtariOS::DSTATS
L_DODET:
	LDA #1
	LDX #DETCRETRY
L_ABS17:
	JMP DOHIDET
; End L_DOIMM



; *************************************************************************************************
; Check for happy 810
L_CHKHAP:
	LDX #$48
	LDA #0
	BEQ L_DOIMM

	.IFNDEF USONLY
	L_SETSPD:
	L_ABS18:
		LDA SIOSPEED
	.ENDIF

L_SETSP2:
	; add 1 to value to prevent endless loop if device sends a speed byte of 0
	CLC
	ADC #1
	JSR L_STOREHSINDEX

	; Fall through ------>
; End L_CHKHAP



; *************************************************************************************************
; ???
L_RESCMD:
	LDY #$00
L_RESLP:
	PLA
	STA AtariOS::DCOMND,Y
	INY
	CPY #$0A
	BCC L_RESLP

	LDA SIOSPEED
	; if speed byte is $FF, drive is offline so just return NAK
	CMP #$FF
	BNE L_ABS20
	LDA #ATARIOS_CIO_STATUS::Timeout
	STA AtariOS::ZP_STATUS
	JMP L_NOWRAP

L_ABS20:
	JMP L_DOSIO
; End L_RESCMD



; *************************************************************************************************
; Retrieve HSIndex for AtariOS::CDEVIC
; Outgoing:
;   Zero flag clear if not found.
;   A is HSIndex + 1 if zero flag is not clear.
;   Y is lost.
L_GETHSINDEX:
	LDY #SPEEDTBSIZE
L_Loop:
	LDA DEVICETB - 1,Y
	CMP AtariOS::CDEVIC
	BEQ L_Found
	DEY
	BNE L_Loop
	RTS
L_Found:
	; found device, so get HSIndex
	LDA SPEEDTB - 1,Y
	RTS
; End L_GETHSINDEX



; *************************************************************************************************
; Store HSIndex for AtariOS::CDEVIC
; Oldest entries are overwritten
; Incoming:
;   A               Byte, The speed.
;   NEXTSPEEDSLOT   Byte, The next speed slot to use. It will be 0 to 3.
; Outgoing:
;   A               Byte, See NEXTSPEEDSLOT below.
;   NEXTSPEEDSLOT   Byte, The original value + 1 & 3, keeping it at 0 to 3.
;   Y is lost.
L_STOREHSINDEX:
	LDY NEXTSPEEDSLOT ; this will be 0-3
	STA SPEEDTB,Y
	LDA AtariOS::CDEVIC
	STA DEVICETB,Y
	INY
	TYA
	AND #SPEEDTBSIZE - 1
	STA NEXTSPEEDSLOT
	RTS
; End L_STOREHSINDEX



; *************************************************************************************************
; 10 bytes, This data is stored in AtariOS::DCOMND to detect Happy/Speedy Ultra SIO mode.
L_CMD3F:
	.BYTE $3F,$40

L_ABS21:
	.WORD HIBUF,2,1,$20
