; highspeed SIO routine, cleanup at end of code
; License: GPL-2, see LICENSE
; This Version (c) 2022 Mark Ormston <memso@memso.net>
; Original copyright (c) 2003-2010 by Matthias Reichl <hias@horus.com>


; *************************************************************************************************
; Cleanup, in case of error or end of transmission.
; Outgoing: All registers are lost.
L_CLEANUP:
	; Reset pokey
	LDA #$A0
	STA POKEY_W::AUDC4

	; Clear command line
	LDA #PIA_CTL::SIO_Output_NegateLine | PIA_CTL::AccessIORegisters
	STA PIA::PBCTL

	; Fall through ------>
; End L_CLEANUP



; *************************************************************************************************
; Fast VBI Inner Functions
.IFDEF FASTVBI
	; =*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=
	; Clear timer 1.
	L_CLRTIM1:
		LDA #$FF
		STA HISIO_MYTIM1 + 1
		RTS
	; End L_CLRTIM1
	


	; =*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=
	; Set timeout
	; Incoming:
	;   X & Y         Word, The time out to use (X high).
	; Outgoing: A is lost.
	L_SMYTOUT:
	L_ABSF06:
		JSR L_WTVBI
		STY HISIO_MYTIM1
		STX HISIO_MYTIM1 + 1
		RTS
	; End L_SMYTOUT
	


	; =*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=
	; Similar to AtariOS::SETVBV: waste 22 cpu cycles after STA ANTIC_W::WSYNC so that the VBI code can be
	; executed if it's the right time for it
	L_WTVBI:
		STA ANTIC_W::WSYNC
		BIT AtariOS::STACKP ; 4
		BIT AtariOS::STACKP ; 4
		BIT AtariOS::STACKP ; 4
		BIT AtariOS::STACKP ; 4
		RTS                 ; 6 (22 total cycles)
	; End L_WTVBI



; *************************************************************************************************
; Non-fast VBI inner functions	
.ELSE
	; =*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=
	; clear timer1
	L_CLRTIM1:
		LDY #$00
		; Fall through ------>
	; End L_CLRTIM1



	; =*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=
	; Set timer1 with Y cycles
	L_STIMOUT:
		LDX #$00
		; Fall through ------>
	; End L_STIMOUT



	; =*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=
	; Set timer1 X*256 + Y cycles
	L_STIMOU2:
		LDA #1
		JSR AtariOS::SETVBV
		.IFDEF RELOCTABLE
		L_ABS65:
			LDA L_ERRTOA
			STA AtariOS::CDTMA1
		L_ABS66:
			LDA L_ERRTOA + 1
			STA AtariOS::CDTMA1 + 1
		.ELSE
			LDA #<(L_ERRTO)
			STA AtariOS::CDTMA1
			LDA #>(L_ERRTO)
			STA AtariOS::CDTMA1 + 1
		.ENDIF
		RTS
	; End L_STIMOU2
.ENDIF



; *************************************************************************************************
; Relocatable data if not using fast VBI
.IFNDEF FASTVBI
	.IFDEF RELOCTABLE
	L_ERRTOA:
	L_ABS67:
		.WORD L_ERRTO
	.ENDIF
.ENDIF
