; highspeed SIO routine, optional diagnostic code
; License: GPL-2, see LICENSE
; This Version (c) 2022 Mark Ormston <memso@memso.net>
; Original copyright (c) 2003-2009 by Matthias Reichl <hias@horus.com>

; the code starting here is only for diagnostic purposes and is
; normally not included

.IFDEF DIAGSIO

; *************************************************************************************************
; Misc Data
L_DIAGS:
	.RES 32, 0
L_DIAGB:
	.RES 32, 0
L_DIAGB2:
	.RES 32, 0
L_DIAGE:
	.RES 32, 0
L_DIAGC:
	.RES 32, 0

; Byte, Used as a loop index in PDIAG.
L_DIAGBT:
	.BYTE 0

L_DIAGX:
	.BYTE 0
L_DIAGST:
	.BYTE 0
L_DIAGER:
	.BYTE 0

	.IFDEF DIAGTIM
	L_TSTAMP:
		.RES 4, 0
	.ENDIF



; *************************************************************************************************
; Prints diagnostic details.
; Incoming:
;   ???
; Outgoing:
;   All registers are lost.
PDIAG:
	; If L_DIAGER is 0, exit
	LDA L_DIAGER
	BNE L_PDIAG0
	RTS


	; ===============================================================================================
	; Displays the low 12 bytes of page 3 (SIO variables) as hex with a space between each
L_PDIAG0:
	LDA #0
	STA L_DIAGBT
L_PDIAG1:
	; Get the offset and exit if it's at 12
	LDX L_DIAGBT
	CPX #12
	BEQ L_PDIAG2

	; Display the value as hex
	LDA $0300,X
	JSR L_PHEX

	; Add the space
	LDA #' '
	JSR L_PUTCHR

	; Next value
	INC L_DIAGBT
	BNE L_PDIAG1

L_PDIAG2:
	; New line
	LDA #ATASCII::EndOfLine
	JSR L_PUTCHR


	; ===============================================================================================
	; Print timestamp if specifed
	.IFDEF DIAGTIM
		LDA L_TSTAMP
		JSR L_PHEX
		LDA L_TSTAMP + 1
		JSR L_PHEX
		LDA L_TSTAMP + 2
		JSR L_PHEX
		LDA #'.'
		JSR L_PUTCHR
		LDA L_TSTAMP + 3
		JSR L_PHEX
		LDA #':'
		JSR L_PUTCHR
		LDA #32
		JSR L_PUTCHR
	.ENDIF


	; ===============================================================================================
	; Displays various diagnostic values
	LDA #0
	STA L_DIAGBT
L_PDIAG3:
	LDX L_DIAGBT
	CPX L_DIAGER
	BEQ L_PDIAG4

	; phase/stage
	LDA L_DIAGS,X
	JSR L_PHEX
	LDA #' '
	JSR L_PUTCHR

	; error/status
	LDX L_DIAGBT
	LDA L_DIAGE,X
	JSR L_PHEX
	LDA #' '
	JSR L_PUTCHR

	; last ack/nak byte
	LDX L_DIAGBT
	LDA L_DIAGB,X
	JSR L_PHEX
	LDA #' '
	JSR L_PUTCHR

	; current byte count
	LDX L_DIAGBT
	LDA L_DIAGC,X
	JSR L_PHEX
	LDA #' '
	JSR L_PUTCHR

	; last received character
	LDX L_DIAGBT
	LDA L_DIAGB2,X
	JSR L_PHEX
	LDA #ATASCII::EndOfLine
	JSR L_PUTCHR

	INC L_DIAGBT
	BNE L_PDIAG3

L_PDIAG4:
	RTS
; End PDIAG



; *************************************************************************************************
; Prints a byte as hex.
; Incoming:
;   A               Byte, The byte to print as hex.
; Outgoing:
;   All registers are lost.
;   I have no idea what changes occur in AtariOS::EOUTCHV.
L_PHEX:
	PHA
	LSR
	LSR
	LSR
	LSR
	TAX
	LDA L_HEXTAB,X
	JSR L_PUTCHR
	PLA
	AND #$0F
	TAX
	LDA L_HEXTAB,X

	; Fall through ------>
; End L_PHEX



; *************************************************************************************************
; Displays a character.
; Incoming:
;   A               Byte, The character to display.
; Outgoing:
;   All registers are lost.
;   I have no idea what changes occur in AtariOS::EOUTCHV.
L_PUTCHR:
	TAY
	LDA AtariOS::EOUTCHV + 1
	PHA
	LDA AtariOS::EOUTCHV
	PHA
	TYA
	RTS
; End L_PUTCHR



; *************************************************************************************************

; 16 bytes, Converts a nybble to a hex character.
L_HEXTAB:
	.BYTE "0123456789ABCDEF"

.ENDIF
