; highspeed SIO routine, optional fast VBI code
; License: GPL-2, see LICENSE
; This Version (c) 2022 Mark Ormston <memso@memso.net>
; Original copyright (c) 2003-2010 by Matthias Reichl <hias@horus.com>

.IFDEF FASTVBI
; *************************************************************************************************
; Fast VBI handler.
VBICODE:
	.IFNDEF FASTVBI_NOCLOCK
		INC AtariOS::ZP_RTCLOK + 2
		BNE L_NOC1
		INC AtariOS::ZP_RTCLOK + 1
		; note: AtariOS::ZP_RTCLOK is handeled in HISIO/Main by checking AtariOS::ZP_RTCLOK + 1 wrap-around
		; this saves 7 cycles in worst case
		;BNE L_NOC1
		;INC AtariOS::ZP_RTCLOK

	L_NOC1:
	.ENDIF

	; Fast implementation of system timer 1:
	; - jumps to L_ERRTO if it reaches 0
	; - no check if timer has already expired (just set HISIO_MYTIM1 + 1 to $FF
	;   and it will run long enough before starting the timeout code)
	; - HISIO_MYTIM1 + 1 has to be set to HIVAL + 1, since timer starts at $0000 (not $FFFF).
	DEC HISIO_MYTIM1
	BNE L_NOTOUT
	DEC HISIO_MYTIM1 + 1

	; timer reached 0, start timeout code
	BEQ L_RUNTIM

L_NOTOUT:

	; note: to save 6 additional cycles we don't use "JMP AtariOS::XITVBV" here
	; but implement the OS code directly. This code is identical in all
	; OS versions I know of.
	PLA
	TAY
	PLA
	TAX
	PLA
	RTI

; jump to timeout code
L_RUNTIM:
L_ABSF07:
	JMP L_ERRTO
; End VBICODE



; *************************************************************************************************
; ???
	.IFDEF RELOCTABLE
	L_VBIADR:
	L_ABSF08:
		.WORD VBICODE
	.ENDIF

.ENDIF
