; 6502 high speed SIO code OS patch program
; License: GPL-2, see LICENSE
; This Version (c) 2022 Mark Ormston <memso@memso.net>
; Original copyright (c) by Matthias Reichl <hias@horus.com>

.INCLUDE "../Include/AtariDOS.inc"
.INCLUDE "../Include/Patch.inc"
.INCLUDE "../Include/CIO.inc"

; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Constants

.IFDEF ROMABLE
PATCHRESET = 1
.ENDIF



; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Addresses

; Temporary ZP vector.
TMPVEC = $E0

; Where this will be located
.IFDEF ATRBOOT
	START = $700
.ELSE
	START = $6000
.ENDIF

; Where the XL SIO code begins.
XLSIO	= $E971

.IFDEF ROMABLE
	ROM1ID = $C000
	ROM2ID = $FFF8

	CADR = $E0
	CEND = $E2
	CSUM = $E4
.ENDIF


; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Start the patch code

.IFDEF ATRBOOT
	.ORG START - 16

	; ATR header
	.WORD $0296 ; magic
	.WORD (MYEND-START + 15)/16
	.WORD 128
	.WORD 0,0,0,0,0

	; boot header
	.BYTE 0
	.BYTE (MYEND-START + 127)/128
	.WORD START
	.WORD EXIT
.ELSE

	; !!! Making it work. This adds a DOS header for the main code
	.ORG START - 6
	AtariDOS_BinaryStart START, MYEND - START
.ENDIF



; *************************************************************************************************
;Start:
	; Display the title message
	BPUT 0, TTITL, TTIT

	; check if OS is already patched
	LDX #3
CHKP:
	LDA XLSIO,X
	CMP MYCODE,X
	BNE CHKP1
	DEX
	BPL CHKP

	PRINT 0, TISP
	JSR CLRSIO

	INC WARNING
	JMP EXIT
; End Start



; *************************************************************************************************
; Clear SIO settings.
CLRSIO:
	LDX #12
	LDA #0
CLRSIOL:
	STA RAMBUF,X
	DEX
	BPL CLRSIOL
	RTS
; End CLRSIO



; *************************************************************************************************
; Continue the patch process.
CHKP1:
	; ===============================================================================================
	; Check if OS is compatible
	LDX #3
CHKCO:
	LDA XLSIO,X
	CMP XLCODE,X
	BNE CHKCO1
	DEX
	BPL CHKCO
	BMI CHKCO2

CHKCO1:
	PRINT 0, TINC
	INC WARNING
	JMP EXIT

CHKCO2:
	.IFDEF ROMABLE
		; check if we need to update the ROM checksums
		JSR CHKCSUM
	.ENDIF

	PRINT 0, TPAT

	; check if we need to move ROM to RAM
	LDA PIA::PORTB
	AND #$01
	STA NEEDRES
	BEQ NOCOPY
	JSR ROMRAM


	; ===============================================================================================
	; Write highspeed SIO code to PDEST
NOCOPY:
	LDX #0
COPLP:
	LDA PCODE,X
	STA PDEST,X
	LDA PCODE + $100,X
	STA PDEST + $100,X
	LDA PCODE + $200,X
	STA PDEST + $200,X
	LDA PCODE + $300,X
	STA PDEST + $300,X
	INX
	BNE COPLP

	; clear speed tab
	JSR CLRSIO


	; ===============================================================================================
	; patch OS SIO routine
	LDX #3
COP2:
	LDA XLSIO,X
	STA PSTD,X
	LDA MYCODE,X
	STA XLSIO,X
	DEX
	BPL COP2

	; ???
	LDA #$4C
	STA PSTD + 4
	LDA #<(XLSIO + 4)
	STA PSTD + 5
	LDA #>(XLSIO + 4)
	STA PSTD + 6


	; ===============================================================================================
	; check if we can patch the keyboard IRQ handler
	.IFDEF PATCHKEY
		LDX #2
	CHKKY:
		LDA KEYIRQ,X
		CMP XLKEY,X
		BNE CHKKY1
		DEX
		BPL CHKKY
		BMI CHKKY2
	
	CHKKY1:
		PRINT 0, TNOKEY
		INC WARNING
		JMP CHKKY4
	
	CHKKY2:
		LDX #2
	CHKKY3:
		LDA MYKEY,X
		STA KEYIRQ,X
		DEX
		BPL CHKKY3
	
		PRINT 0, TKEY
	
	CHKKY4:
	.ENDIF


	; ===============================================================================================
	; check if we can patch the powerup/reset code	
	.IFDEF PATCHRESET
		LDX #2
	CHKBT:
		LDA PUPCODE,X
		CMP XLPUP,X
		BNE CHKBT1
		DEX
		BPL CHKBT
		BMI CHKBT2
	
	CHKBT1:
		PRINT 0, TNOPUP
		INC WARNING
		JMP CHKBT4
	
	CHKBT2:
		LDX #2
	CHKBT3:
		LDA MYPUP,X
		STA PUPCODE,X
		DEX
		BPL CHKBT3
	
		PRINT 0, TPUP
	
	CHKBT4:
	.ENDIF


	.IFDEF ROMABLE
		; update ROM checksums
		JSR UPDCSUM
	.ENDIF


	; ===============================================================================================
	; Install the reset handler unless it isn't needed

	; Skip if the reset handler isn't needed
	LDA NEEDRES
	BEQ NORES

	; install reset handler
	LDX #RESETL - 1
COPRES:
	LDA RESET,X
	STA RESETAD,X
	DEX
	BPL COPRES

	; Set up the reset address
	LDA #<(RESETAD)
	STA AtariOS::ZP_CASINI
	LDA #>(RESETAD)
	STA AtariOS::ZP_CASINI + 1

	; OR bit 1 in AtariOS::ZP_BOOTQ
	LDA AtariOS::ZP_BOOTQ
	ORA #2
	STA AtariOS::ZP_BOOTQ


	; ===============================================================================================
	; Done
NORES:
	JMP EXIT
; End CHKP1



; *************************************************************************************************
; Data

; Byte, Set to non-zero to replace the reset code.
NEEDRES:
	.BYTE 0

; Byte, ??? - I would guess it's just non-zero if there is a warning of some sort
WARNING:
	.BYTE 0

.INCLUDE "ROMToRAM.asm"

.IFDEF ATARISIO_SWAP
	.INCLUDE "AtariSIO.asm"
.ENDIF



; *************************************************************************************************
; reset routine: 
RESET:
	; Do a coldstart if SHIFT is pressed
	LDA POKEY_R::SKSTAT
	AND #8
	BNE L_NOCOLD
	JMP AtariOS::COLDSV
L_NOCOLD:

	; Set the OS ROM to disabled
	LDA PIA::PORTB
	AND #$FE
	STA PIA::PORTB

	RTS
; End RESET
RESETL	= * - RESET



; *************************************************************************************************
; ??? - Obviously, it's named EXIT... but EXIT what?
EXIT:
	; $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
	; ATRBOOT version
	.IFDEF ATRBOOT
		.IFDEF ATARISIO_SWAP
			JSR SWAP_DRIVES
			BCC EXITX
		.ENDIF

		; Display the boot message
		BPUT 0, TBOOTL, TBOOT

	WCONSOL:
		LDA GTIA_R::CONSOL
		CMP #6
		BNE WCONSOL
		LDA #ATASCII::EndOfLine
		JSR PUTCHR
	
	EXITX:
		LDA #<(START)
		STA TMPVEC
		LDA #>(START)
		STA TMPVEC + 1
		LDY #CLEANUPL - 1

	CPCLUP:
		LDA CLEANUP,Y
		STA RESETAD+RESETL,Y
		DEY
		BPL CPCLUP
		LDA #0
		TAY
		LDX #>(MYEND - START + 255)
		JMP RESETAD+RESETL
	

	; ===============================================================================================
	; Inner function: Clean up. Stores a value in the temporary vector's target over a 16-bit number
	; of values.
	; Incoming:
	;   A             Byte, The value to store.
	;   X & Y         Word, The length of the data to set to A. (X high)
	;   TMPVEC        Address, Where to clear the data.
	; Outgoing:
	;   Carry is set.
	;   X and Y are lost.
	;   TMPVEC is lost.
	CLEANUP:
		STA (TMPVEC),Y
		INY
		BNE CLEANUP
		INC TMPVEC + 1
		DEX
		BNE CLEANUP
		SEC
		RTS
	; End CLEANUP
	CLEANUPL	= * - CLEANUP
	

	; ===============================================================================================
	; Inner function: Display the character in A.
	; Incoming:
	;   A             Byte, The character to display.
	; Outgoing:
	;   All registers are lost.
	PUTCHR:
		TAY
		LDA AtariOS::EOUTCHV + 1
		PHA
		LDA AtariOS::EOUTCHV
		PHA
		TYA
		RTS
	; End PUTCHR


	; $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
	; Other version
	.ELSE
		; Delay for approx. 2 seconds if some warning occurred
		LDA WARNING
		BEQ NODELAY
		CLC
		LDA AtariOS::ZP_RTCLOK + 2
		ADC #100
	W20:
		CMP AtariOS::ZP_RTCLOK + 2
		BNE W20

	NODELAY:
		RTS
	.ENDIF
; End EXIT



; *************************************************************************************************
; Though this is technically code, it is used to test if the original SIO code is in memory or not.
; This isn't actually called.
XLCODE:
	TSX
	STX AtariOS::STACKP
; End XLCODE



; *************************************************************************************************
; Though this is technically code, it is copied over the first 4 bytes of the XL's SIO code and not
; actually called here.
MYCODE:
	NOP
	JMP PSIO
; End MYCODE



; *************************************************************************************************
; Though this is technically code, it is used to test if the original XL keyboard handler exists.
.IFDEF PATCHKEY
	; Original contents of keyboard IRQ handler
	XLKEY:
		LDA POKEY_R::KBCODE

	; Patched contents of keyboard IRQ handler
	MYKEY:
		JSR PKEY
.ENDIF



; *************************************************************************************************
; Though this is technically code, it is used to test if the original XL reset code exists.
.IFDEF PATCHRESET
	; Original contents of reset code
	XLPUP:
		LDA $033D

	; Patched contents of reset code
	MYPUP:
		JMP PPWRUP
.ENDIF



; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Romable extras
.IFDEF ROMABLE
	; ***********************************************************************************************
	; Calculate checksum of a ROM-block
	CSBLK:
		LDY #0
	CSSET:
		LDA ADRTAB,X
		STA CADR,Y
		INX
		INY
		CPY #4
		BNE CSSET
		LDY #0
	CSBLKL:
		CLC
		LDA (CADR),Y
		ADC CSUM
		STA CSUM
		BCC CSBLKL1
		INC CSUM + 1
	CSBLKL1:
		INC CADR
		BNE CSBLKL2
		INC CADR + 1
	CSBLKL2:
		LDA CADR
		CMP CEND
		BNE CSBLKL
		LDA CADR + 1
		CMP CEND + 1
		BNE CSBLKL
		RTS
	; End CSBLK
	


	; ***********************************************************************************************
	; Checksum of first 8k ROM	
	CHK1:
		LDA #0
		STA CSUM
		STA CSUM + 1
		TAX
		JSR CSBLK
	
		LDA $D301
		PHA
		AND #$7F
		ORA #1
		STA $D301
		JSR CSBLK
	
		PLA
		STA $D301
		JSR CSBLK
	
		RTS
	; End CHK1
	


	; ***********************************************************************************************
	; Checksum of second 8k ROM	
	CHK2:
		LDA #0
		STA CSUM
		STA CSUM + 1
		LDX #12
		JSR CSBLK
		JSR CSBLK
		RTS
	; End CHK2



	; ***********************************************************************************************
	; Data
	
	ADRTAB:
		.WORD $C002,$D000
		.WORD $5000,$5800
		.WORD $D800,$E000
		.WORD $E000,$FFF8
		.WORD $FFFA,$0000
	
	DOCSUPD:
		.BYTE 0
	


	; ***********************************************************************************************
	; Checks the checksum.
	CHKCSUM:
		LDA #0
		STA DOCSUPD
		JSR CHK1
		LDA ROM1ID
		CMP CSUM
		BNE CSERR
		LDA ROM1ID + 1
		CMP CSUM + 1
		BNE CSERR
	
		JSR CHK2
		LDA ROM2ID
		CMP CSUM
		BNE CSERR
		LDA ROM2ID + 1
		CMP CSUM + 1
		BNE CSERR
	
		INC DOCSUPD
		RTS
	
	CSERR:
		; Display the checksum error message
		BPUT 0, TCSERRL, TCSERR
		RTS
	; End CHKCSUM
	


	; ***********************************************************************************************
	; Update the checksum
	UPDCSUM:
		LDA DOCSUPD
		BEQ UPDCSE
	
		PRINT 0, TUPDCS
	
		JSR CHK1
		LDA CSUM
		STA ROM1ID
		LDA CSUM + 1
		STA ROM1ID + 1
	
		JSR CHK2
		LDA CSUM
		STA ROM2ID
		LDA CSUM + 1
		STA ROM2ID + 1
	
	UPDCSE:
		RTS
	; End UPDCSUM
	


	; ***********************************************************************************************
	TCSERR:
		.BYTE "ROM checksum error - skipping",ATASCII::EndOfLine
		.BYTE "checksum update", ATASCII::EndOfLine,ATASCII::EndOfLine
	TCSERRL	= * - TCSERR
	
	TUPDCS:
		.BYTE "updating ROM checksum",ATASCII::EndOfLine
.ENDIF



; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Various Strings

.IFDEF PATCHKEY
TNOKEY:
	.BYTE "unknown ROM, not patching keyboard",ATASCII::EndOfLine
TKEY:
	.BYTE "patched keyboard IRQ handler",ATASCII::EndOfLine
.ENDIF

.IFDEF PATCHRESET
TNOPUP:
	.BYTE "unknown ROM, not patching powerup code",ATASCII::EndOfLine
TPUP:
	.BYTE "patched powerup code",ATASCII::EndOfLine
.ENDIF

TTIT:
.IFDEF ROMABLE
	.IFDEF ATRBOOT
		.BYTE 125
	.ELSE
		.BYTE ATASCII::EndOfLine
	.ENDIF
		.BYTE "Highspeed SIO patch V1.32R",ATASCII::EndOfLine
.ELSE
	.IFDEF ATRBOOT
		.BYTE 125
	.ELSE
		.BYTE ATASCII::EndOfLine
	.ENDIF
	.BYTE "Highspeed SIO patch V1.32",ATASCII::EndOfLine
.ENDIF
	.BYTE "(c) 2020 by Matthias Reichl",ATASCII::EndOfLine,ATASCII::EndOfLine
TTITL	= * - TTIT

.IFDEF ATRBOOT
	TBOOT:
		.BYTE ATASCII::EndOfLine,"Press START to boot disk"
	TBOOTL	= * - TBOOT
.ENDIF

TISP:
	.BYTE "OS already patched",ATASCII::EndOfLine
TINC:
	.BYTE "incompatible OS",ATASCII::EndOfLine
TPAT:
	.BYTE "patching OS",ATASCII::EndOfLine



; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Include the main patch code
PCODE	= *

.IFDEF ROMABLE
	.INCBIN "../Output/Main-PatchCode-ROM.bin"
.ELSE
	.INCBIN "../Output/Main-PatchCode.bin"
.ENDIF

; Pad to a 128-byte boundary if making an ATR file
.IFDEF ATRBOOT
	.IF (* & 127) <> 0
		.RES (128 - (* & 127)), 0
	.ENDIF
.ENDIF
MYEND	= *



; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Add the DOS segment that sets the START address
.IFNDEF ATRBOOT
	; !!! Making it work. Adds the start vector for DOS
	.ORG AtariDOS::RUNAD
	AtariDOS_BinaryBlock AtariDOS::RUNAD, 2
	.WORD START
.ENDIF
