; 6502 high speed SIO code OS patch
; License: GPL-2, see LICENSE
; This Version (c) 2022 Mark Ormston <memso@memso.net>
; Original copyright (c) 2006-2009 by Matthias Reichl <hias@horus.com>
;
; Changes:
; 2008-06-26 Matthias Reichl
;   - add (optional) keyboard IRQ patch
;
; 2009-02-10 Matthias Reichl
;   - added NMI patch (for insane speeds)
;
; 2009-05-01 Matthias Reichl
;   - added coldstart/powerup patch: SHIFT+RESET does a coldstart
;   - rearranged memory layout to fit code into $CCxx
;
; 2010-08-14 Matthias Reichl
;   - removed NMI patch (obsoleted by new fast VBI code)

.INCLUDE "../Include/Patch.inc"

; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; ???
.ORG PDEST
	.BYTE 0

	; patch program must fill this area with jump to standard SIO routine
	STDSIO = PSTD

; !!! Makes this match the original output
.RES 23, 0



; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
.ORG PPWRUP

; *************************************************************************************************
; powerup/reset code: do a cold boot if SHIFT is pressed
;PPWRUP:
	; Enable keyboard scan and debounce
	LDA #3
	STA POKEY_W::SKCTL

	; Get the keyboard SHIFT key and if a standard key has been pressed
	LDA POKEY_R::SKSTAT
	AND #$0C

	; Test if SHIFT is pressed and no other key. If so, do a cold boot.
	EOR #4
	BEQ L_DOCOLD

	; original instruction at $C3B3, was patched with "JMP PBOOT"
	LDA $033D

L_DOCOLD:
	JMP PUPCODE + 3
; End PPWRUP

; !!! Makes this match the output
.RES 4, 0



; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; ???
.ORG PSIO

; *************************************************************************************************
; ???
;PSIO:	
	.IFDEF PATCHKEY
		LDA HIENAB
		BNE STDSIO
	.ENDIF

	.IFDEF BT
		.INCLUDE "../Main/DetectBT.asm"
	.ELSE
		LDA AtariOS::DDEVIC
		CMP #$31
		BNE STDSIO
		LDA AtariOS::DUNIT
		BEQ STDSIO
		CMP #9
		BCS STDSIO
	
		.INCLUDE "../Main/Detect.asm"
	.ENDIF

	.INCLUDE "../Main/MergeHISIO.asm"
; End PSIO

; !!! Makes this match the original
.IFDEF PATCHKEY
	.RES 17, 0
.ENDIF



; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; ???
.IFDEF PATCHKEY
.ORG PKEY

; *************************************************************************************************
; Tests for special key presses.
;PKEY:
	; ===============================================================================================
	; Was Shift-Control-Delete pressed?
	LDA POKEY_R::KBCODE
	CMP #$F4
	BNE PKEY1

	; Cold start
	JMP AtariOS::COLDSV


	; ===============================================================================================
	; Was Shift-Control-S pressed?
PKEY1:
	PHA
	CMP #$FE
	BNE PKEY3

	; Reset all variables
	TXA
	PHA
	LDA #0
	; HIENAB must be directly after speed table!
	LDX #8
PKEY2:
	; clear drive speed table
	STA SPEEDTB,X
	DEX
	BPL PKEY2

	PLA
	TAX

	JMP PKEYX


	; ===============================================================================================
	; Was Shift-Control-H pressed?
PKEY3:
	CMP #$F9
	BNE PKEY4

	; Enable high speed SIO
	LDA #0
	STA HIENAB
	BEQ PKEYX


	; ===============================================================================================
	; Was Shift-Control-N pressed?
PKEY4:
	CMP #$E3
	BNE PKEYX

	; Enable normal speed SIO
	LDA #$FF
	STA HIENAB
	
PKEYX:
	PLA
	RTS
; End PKEY
.ENDIF

; !!! Makes this match the original output
.IFDEF BT
	.RES 9, 0
.ENDIF



; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; ???
.ORG PHISIO

JMP DOHIENTRY



; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; ???
.ORG PVER
	.BYTE "Hias 1.32 "
	.IFDEF BT
		.BYTE "SIO2BT"
	.ELSE
		.BYTE "200428"
	.ENDIF
