; Copy OS ROM to RAM
; License: GPL-2, see LICENSE
; This Version (c) 2022 Mark Ormston <memso@memso.net>
; Original Copyright (c) 2009 Matthias Reichl <hias@horus.com>
; NOTE: You may need to modify TMPVEC to some other zero page address.

.INCLUDE "../Include/HISIO.inc"

; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Addresses

; Address, A zero page pointer. This may go anywhere in the zero page other than $00 - $81 and $FF.
.IFNDEF TMPVEC
	TMPVEC = $E0
.ENDIF



; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Code

; *************************************************************************************************
; Disables interrupts and copies the Atari ROM to RAM. This assumes that PIA::PORTB is in its default
; state and has $FF in it.
ROMRAM:
	; Clear interrupts, ANTIC_W::NMIEN, ANTIC_W::DMACTL
	SEI
	LDY #0
	STY ANTIC_W::NMIEN
	STY ANTIC_W::DMACTL

	; Set TMPVEC to $C000
	STY TMPVEC
	LDA #$C0
	STA TMPVEC + 1

	; Copy a page by reading, changing to RAM, writing, changing back to ROM
L_ROMMV:
	LDA (TMPVEC),Y
	DEC PIA::PORTB
	STA (TMPVEC),Y
	INC PIA::PORTB
	INY
	BNE L_ROMMV

	; Move to the next page and exit if done
	INC TMPVEC + 1
	LDA TMPVEC + 1
	BEQ L_ROMOK

	; If it hit $D0, skip past the Atari hardware to $D8, then repeat
	CMP #$D0
	BNE L_ROMMV
	LDA #$D8
	STA TMPVEC + 1
	BNE L_ROMMV

L_ROMOK:
	; Set the high memory to RAM
	DEC PIA::PORTB

	; Reenable interrupts
	LDA #$40
	STA ANTIC_W::NMIEN
	CLI

	RTS
; End ROMRAM
