; Highspeed SIO diagnostics
; License: GPL-2, see LICENSE
; This Version (c) 2022 Mark Ormston <memso@memso.net>
; Original copyright (c) 2003-2009 by Matthias Reichl <hias@horus.com>

; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Constants

; 0 for keyboard pauses, 1 for no keyboard pauses.
NOKEY = 1

.IFDEF DIAG_WRITE
	TCMD = 'P'
	TDIR = $80
	RNDBUF = 1
.ELSE
	TCMD = 'R'
	TDIR = $40
.ENDIF

.IFNDEF SHIPDIAG
	; ------------------------------------------------------------
	; begin: my own testing parameters


	; disable retries and fallback to standard speed
	; (even in non-diag mode)
	; DIAGNORET = 1
	; DIAGTX = 1
	; diag specific timing problems
	; DIAGTP = 1

	DIAGNORET = 1
	DIAGTX = 1
	;DIAGSIO = 1
	;DIAGTIM = 1
	FASTVBI = 1

	;DIAGTP = 1
	;CLK1 = $FF
	;CLK2 = $FF
	;CLK3 = $FF

	;LCNT = $56	; causes timeout with 94 NMI cycles
	;DELCNT = 5

	;LCNT = $56
	;LCNT = $85
	;LCNT = $15
	;LCNT1 = 1 ; add one "STA ANTIC_W::WSYNC"

	; use a fixed timout value, for worst-case testing
	;FIXTOUT = $201

	;FASTVBI_NOCLOCK = 1

	; use immediate VBI ($222) instead of patching NMI handler
	;FASTVBI = 1

	; DIAGSIO = 1
	; DIAGTIM = 1

	; DMADEF=0 ; screen off
	; DMADEF=$23 ; wide playfield
	;DMADEF=$26 ; normal PF, missile DMA
	;DMADEF=$36 ; normal PF, missile DMA, single line resolution
	;DMADEF=$2A ; normal PF, player DMA

	; SCREENOFF = 1

	; clear buffer before every SIO call
	; CLRBUF=1

	; DIAGSIO = 1

	; diag timing (in highspeed code)
	; DIAGTIM = 1

	; NONMI = 1

	; USEDLI = 1

	; don't do checksumming
	; NOCKSUM = 1

	; diag timing (out of highspeed code)
	;DIAGTX = 1


	; initial clock values
	; CLK1 = $FF
	; CLK2 = $FF
	; CLK3 = $FF

	; lcount value
	;LCNT = $8E

	; short additional delay loop: DELCNT * 5 - 1 cycles
	;DELCNT = 80

	DRIVENO = 2

	; store random data in buffer
	;RNDBUF = 1

	;TBYT = $80
	TBYT = $100

	; end: my own testing parameters
	; ------------------------------------------------------------

.ELSE
	; default testing parameters, included for testing in dist
	DRIVENO = 2

	;TBYT = $80
	TBYT = $100

	.IF SHIPDIAG=1
		; do timing tests with unmodified SIO code
		DIAGNORET = 1
		DIAGTX = 1
		FASTVBI = 1
	.ELSEIF SHIPDIAG=2
		; test with all diags in SIO code
		DIAGSIO = 1
		DIAGTIM = 1
		FASTVBI = 1
	.ENDIF

.ENDIF

; Byte, The value to pass to ANTIC_W::NMIEN. If USEDLI is set, allow DLI interrupts and VBIs. Otherwise, just VBIs.
.IFDEF USEDLI
	DEFNMI = $C0
.ELSE
	DEFNMI = $40
.ENDIF



; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Includes
.INCLUDE "../Include/CIO.inc"
.INCLUDE "../Include/HISIO.inc"



; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Addresses

; *************************************************************************************************
; Zero Page

; Byte, ???. MSKRECV is stored here.
ZP_UnknownA0 = $A0

; Address, A zero page pointer to use.
TMPVEC = $E0



; *************************************************************************************************
; Others

; Byte, ???. This is memory that is guaranteed unused by the OS. MSKRECV is stored here.
Unknown0600 = $0600

; ???, Some sort of buffer.
BUF = $0800

; Where the start of the code is.
START = $1000



; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Code and Data
.ORG START - 16

; *************************************************************************************************
; ATR file header
	.WORD $0296 ; magic
	.WORD (END-START + 15)/16
	.WORD 128
	.RES 10, 0



; *************************************************************************************************
; Boot header complete file
;START: (defined above)
	.BYTE 0
	.BYTE (END-START + 127)/128
	.WORD START
	.WORD XRTS



; *************************************************************************************************
; Boot Start
BootStart: ; Never referenced
	LDA #MSKRECV
	STA ZP_UnknownA0
	STA Unknown0600

	LDA #0
	STA AtariOS::COLDST
	LDA #2
	STA AtariOS::ZP_BOOTQ
	LDA #<(RUN)
	STA AtariOS::ZP_CASINI
	LDA #>(RUN)
	STA AtariOS::ZP_CASINI + 1
	; Fall through ------>
; End BootStart



; *************************************************************************************************
; The main run
RUN:
	LDA #0
	STA AtariOS::DUNIT
	JSR HISIO ; clear speed table


	; ===============================================================================================
	; Prepare if using a DLI
	.IFDEF USEDLI
		LDA #0
		STA ANTIC_W::NMIEN

		; Set up the DLI
		LDA #<(DLI)
		STA AtariOS::VDSLST
		LDA #>(DLI)
		STA AtariOS::VDSLST + 1

		; Copy the saved display list
		LDA Atari::SDLSTL
		STA TMPVEC
		LDA Atari::SDLSTL + 1
		STA TMPVEC + 1
		.IF 0
			LDY #2
			LDA (TMPVEC),Y
			ORA #$80
			STA (TMPVEC),Y
			INY
			LDA (TMPVEC),Y
			ORA #$80
			STA (TMPVEC),Y

			LDY #6
		L_DLILP:
			LDA (TMPVEC),Y
			ORA #$80
			STA (TMPVEC),Y
			INY
			CPY #28
			BNE L_DLILP
		.ENDIF

		LDY #10
		LDA (TMPVEC),Y
		ORA #$80
		STA (TMPVEC),Y

		LDA #DEFNMI
		STA ANTIC_W::NMIEN
	.ENDIF


	; ===============================================================================================
	; Set SIO parameters for D2:
	LDA #$31
	STA AtariOS::DDEVIC
	LDA #DRIVENO
	STA AtariOS::DUNIT
	LDA #7
	STA AtariOS::DTIMLO


	; =====================================================================================
	; Read sectors 4-720 in a loop forever.
DOTEST: ; This is never referenced

	; print menu screen
	; drive number
	LDA #DRIVENO + $30
	STA MDRIV

	; density
	.IF TBYT = $80
		LDA #'S'
	.ELSE
		LDA #'D'
	.ENDIF
	STA MDENS

	; ???
	BPUT 0, MLEN, MTXT

	.IFNDEF NOKEY
		; Get a keypress and call cold start if escape is pressed
		JSR GETKEY
		CMP #27
		BNE NOBOOT
		JMP AtariOS::COLDSV
	.ENDIF

NOBOOT:
	LDA #TCMD
	STA AtariOS::DCOMND
	LDA #<(TBYT)
	STA AtariOS::DBYT
	LDA #>(TBYT)
	STA AtariOS::DBYT + 1

	LDA #<(BUF)
	STA AtariOS::DBUF
	LDA #>(BUF)
	STA AtariOS::DBUF + 1

	; reset run-counter
	LDA #0
	STA RUNCNT
	STA RUNCNT + 1
	STA RUNCNT + 2

	.IFDEF DMADEF
		LDA #DMADEF
		STA AtariOS::SDMCTL
		STA ANTIC_W::DMACTL
	.ENDIF

TLOOP0:
	LDA #4
	STA AtariOS::DAUX1
	LDA #0
	STA AtariOS::DAUX2

TLOOP:
	LDA GTIA_R::CONSOL
	CMP #6
	BEQ TLOOP

	INC RUNCNT
	BNE RUN1
	INC RUNCNT + 1
	BNE RUN1
	INC RUNCNT + 2
RUN1:

	LDA #TDIR
	STA AtariOS::DSTATS

	; Clear NMIs
	LDA #0
	STA ANTIC_W::NMIEN

	.IFDEF CLRBUF
		; Set BUF to 256 0s.
		LDA #0
		TAY
	CLRLP:
		STA BUF,Y
		INY
		BNE CLRLP

	.ELSEIF .DEF(RNDBUF)
		; Set BUF to 256 random bytes.
		LDY #0
	CLRLP:
		LDA POKEY_R::RANDOM
		STA BUF,Y
		INY
		BNE CLRLP
	.ENDIF

	.IFDEF DIAGTP
		LDA #CLK3
		STA AtariOS::ZP_RTCLOK + 2
		LDA #CLK2
		STA AtariOS::ZP_RTCLOK + 1
		LDA #CLK1
		STA AtariOS::ZP_RTCLOK

		; now wait for the right screen line
		.IFDEF LCNT
			LDX #LCNT - 1
		L_WTLC1:
			CPX ANTIC_R::VCOUNT
			BNE L_WTLC1
			INX
		L_WTLC2:
			CPX ANTIC_R::VCOUNT
			BNE L_WTLC2
			.IFDEF LCNT1
				STA ANTIC_W::WSYNC
			.ENDIF
		.ENDIF

		.IFDEF DELCNT
			LDX #DELCNT
		L_WTDEL:
			DEX
			BNE L_WTDEL
		.ENDIF
	.ENDIF

	.IFDEF DIAGTX
		; record current timestamp
	  LDA AtariOS::ZP_RTCLOK
	  STA TSTAMP
	  LDA AtariOS::ZP_RTCLOK + 1
	  STA TSTAMP + 1
	  LDA AtariOS::ZP_RTCLOK + 2
	  STA TSTAMP + 2
	  LDA ANTIC_R::VCOUNT
	  STA TSTAMP + 3
	.ENDIF

	.IFNDEF NONMI
		LDA #DEFNMI
		STA ANTIC_W::NMIEN
	.ENDIF

	JSR HISIO

	.IFDEF DIAGSIO
		JSR PDIAG
	.ENDIF

	LDA GTIA_R::CONSOL
	CMP #3
	BEQ TEND

	LDA AtariOS::DSTATS
	BMI TEND

	INC AtariOS::DAUX1
	BNE TLOOP1
	INC AtariOS::DAUX2
TLOOP1:
	LDA AtariOS::DAUX2
	CMP #2
	BNE TLOOP
	LDA AtariOS::DAUX1
	CMP #$D1
	BNE TLOOP
	JMP TLOOP0

TEND:
	LDA #$22
	STA AtariOS::SDMCTL
	LDA #DEFNMI
	STA ANTIC_W::NMIEN

	BPUT 0, TRUNL, TRUN
	LDA RUNCNT + 2
	JSR PHEX
	LDA RUNCNT + 1
	JSR PHEX
	LDA RUNCNT
	JSR PHEX
	LDA #ATASCII::EndOfLine
	JSR PUTCHR

	.IFDEF DIAGTX
		LDA TSTAMP
		JSR PHEX
		LDA TSTAMP + 1
		JSR PHEX
		LDA TSTAMP + 2
		JSR PHEX
		LDA #'.'
		JSR PUTCHR
		LDA TSTAMP + 3
		JSR PHEX
		LDA #':'
		JSR PUTCHR
		LDA #' '
		JSR PUTCHR
		LDA AtariOS::DSTATS
		JSR PHEX
		LDA #ATASCII::EndOfLine
		JSR PUTCHR
	.ENDIF

TEND2:
	BPUT 0, TAGAL, TAGA

	; Get a keypress and do a cold start if escape is pressed
	JSR GETKEY
	CMP #27
	BNE NOBOOT2
	JMP AtariOS::COLDSV

NOBOOT2:
	JMP RUN



; *************************************************************************************************
; On start, the Atari booter JSRs to here. This just does an RTS.
XRTS:
	RTS



; *************************************************************************************************
; Calls PUTCHR with the two hex digits of a byte.
; Incoming:
;   A               Byte, The value to convert to hex.
PHEX:
	PHA
  LSR
  LSR
  LSR
  LSR
  TAX
  LDA HEXTAB,X
  JSR PUTCHR
  PLA
  AND #$0F
  TAX
  LDA HEXTAB,X
  ; Fall through ------>
; End PHEX



; *************************************************************************************************
; Calls AtariOS::EOUTCHV which seems to be some sort of "put character".
; Incoming:
;   A               Byte, The value to output.
PUTCHR:
	TAY
  LDA AtariOS::EOUTCHV + 1
  PHA
  LDA AtariOS::EOUTCHV
  PHA
  TYA
  RTS
; End PUTCHR



; *************************************************************************************************

; 16 bytes, Table of values to hex characters.
HEXTAB:
	.BYTE "0123456789ABCDEF"

TSTAMP:
	.RES 4, 0



; *************************************************************************************************
; Calls the OS to get a keypress.
GETKEY:
	LDA AtariOS::KGETCHV + 1
	PHA
	LDA AtariOS::KGETCHV
	PHA
	RTS
; End GETKEY



; *************************************************************************************************
; A very minimal display list interrupt.
	.IFDEF USEDLI
	DLI:
		PHA
		LDA ANTIC_R::VCOUNT
		STA ANTIC_W::WSYNC
		STA GTIA_W::COLBK
		PLA
		RTI
	.ENDIF



; *************************************************************************************************
; Various sets of content to use

; =================================================================================================
MTXT:
	.BYTE 125
	.BYTE "highspeed SIO test 2018-10-30",ATASCII::EndOfLine
	.IFDEF SHIPDIAG
		.IF SHIPDIAG = 1
			.BYTE "basic "
			.IFDEF DIAG_WRITE
				.BYTE "write "
			.ELSE
				.BYTE "read "
			.ENDIF
			.BYTE "test, fast VBI",ATASCII::EndOfLine
		.ELSEIF SHIPDIAG = 2
			.BYTE "extended "
			.IFDEF DIAG_WRITE
				.BYTE "write "
			.ELSE
				.BYTE "read "
			.ENDIF
			.BYTE "test, fast VBI",ATASCII::EndOfLine
		.ENDIF
	.ELSE
		.BYTE "custom "
		.IFDEF DIAG_WRITE
			.BYTE "write "
		.ELSE
			.BYTE "read "
		.ENDIF
		.BYTE "test",ATASCII::EndOfLine
	.ENDIF

	.IFDEF NOKEY
		.BYTE ATASCII::EndOfLine, "testing "
	MDENS:
		.BYTE "SD disk in D"
	MDRIV:
		.BYTE "0:",ATASCII::EndOfLine
		.BYTE ATASCII::EndOfLine, "press OPTION to stop test"

	.ELSE
		.BYTE ATASCII::EndOfLine,"insert "
	MDENS:
		.BYTE "SD disk into D"
	MDRIV:
		.BYTE "0: and",ATASCII::EndOfLine
		.BYTE "press any key, ESC=reboot"
	.ENDIF
	.BYTE ATASCII::EndOfLine,ATASCII::EndOfLine

MLEN = * - MTXT


; =================================================================================================
TAGA:
	.BYTE ATASCII::EndOfLine, "press ESC to reboot or any",ATASCII::EndOfLine
	.BYTE "other key to restart test",ATASCII::EndOfLine
TAGAL = * - TAGA


; =================================================================================================
TRUN:
	.BYTE "runs: "
TRUNL = * - TRUN


; =================================================================================================
RUNCNT:
	.BYTE 0,0,0



; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; assemble highspeed code at identical page-boundaries as hipatch code

.IF (* & 255)<>0
	.RES (256 - (* & 255)), 0
.ENDIF

HIBUF:
	.RES 4, 0
SIOSPEED:
	.BYTE 0
SPEEDTB:
	.RES 8, 0
	.RES 19, 0
	.RES 21, 0

.ORG $1335

.INCLUDE "../Main/Detect.asm"
.INCLUDE "../Main/MergeHISIO.asm"

.IF (* & 127)<>0
	.RES (128 - (* & 127)), 0
.ENDIF

END:
