; Highspeed SIO patch entry point test
; License: GPL-2, see LICENSE
; This Version (c) 2022 Mark Ormston <memso@memso.net>
; Original copyright (c) by Matthias Reichl <hias@horus.com>

.INCLUDE "../Include/HISIO.inc"
.INCLUDE "../Include/Patch.inc"
.INCLUDE "../Include/CIO.inc"

; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Addresses

; Address, A zero page value to store a temporary vector to.
TMPVEC = $E0

; Where the program starts.
START	= $1000

; Where the data buffer is located.
BUF	= $0800



; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Constants
DRIVENO = 2
TBYT	= $100
TCMD	= 'R'
TDIR	= $40

; highspeed SIO mode to use
HIMODE	= $08



; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Code & Data
.ORG START - 16

; *************************************************************************************************
; ATR file header
	.WORD $0296 ; magic
	.WORD (END-START + 15)/16
	.WORD 128
	.WORD 0,0,0,0,0



; *************************************************************************************************
; boot header complete file
	.BYTE 0
	.BYTE (END-START + 127)/128
	.WORD START
	.WORD XRTS
       


; *************************************************************************************************
; check for highspeed patch presence in OS
MAIN:
	; Display the main text
	BPUT 0, MTXTL, MTXT

	; ===============================================================================================
	; Checks PVER for /Hias [0-9]\.[0-9]{2} /
	LDA PVER
	CMP #'H'
	BNE NOPATCH

	LDA PVER + 1
	CMP #'i'
	BNE NOPATCH

	LDA PVER + 2
	CMP #'a'
	BNE NOPATCH

	LDA PVER + 3
	CMP #'s'
	BNE NOPATCH

	LDA PVER + 4
	CMP #' '
	BNE NOPATCH

	LDA PVER + 5
	CMP #'0'
	BCC NOPATCH
	CMP #'9' + 1
	BCS NOPATCH

	LDA PVER + 6
	CMP #'.'
	BNE NOPATCH

	LDA PVER + 7
	CMP #'0'
	BCC NOPATCH
	CMP #'9' + 1
	BCS NOPATCH

	LDA PVER + 8
	CMP #'0'
	BCC NOPATCH
	CMP #'9' + 1
	BCS NOPATCH

	LDA PVER + 9
	CMP #' '
	BEQ FOUNDPATCH


	; ===============================================================================================
	; Patch not found
NOPATCH:
	; Display the no patch message
	BPUT 0, TNOPATCHL, TNOPATCH

	; End
	JMP TEND


	; ===============================================================================================
	; Found the patch
FOUNDPATCH:
	; Display the patch found message
	BPUT 0, TFOUNDL, TFOUND

	; Display the version
	BPUT 0, 11, PVER + 5
	LDA #ATASCII::EndOfLine
	JSR PUTCHR

	; now check if version >= 1.32
	LDA PVER + 5
	CMP #'1'
	BCC OLDPATCH
	BNE PATCHOK

	LDA PVER + 7
	CMP #'3'
	BCC OLDPATCH
	BNE PATCHOK

	LDA PVER + 8
	CMP #'2'
	BCS PATCHOK

OLDPATCH:
	; Old patch, display a message about it being too old and exit
	BPUT 0, TOLDL, TOLD
	JMP TEND

PATCHOK:
	; Good enough version found, display an OK message
	BPUT 0, TVEROKL, TVEROK


	; ===============================================================================================
	; Set SIO parameters
	LDA #'1'
	STA AtariOS::DDEVIC
	LDA #DRIVENO
	STA AtariOS::DUNIT
	LDA #TCMD
	STA AtariOS::DCOMND
	LDA #<(BUF)
	STA AtariOS::DBUF
	LDA #>(BUF)
	STA AtariOS::DBUF + 1
	LDA #7
	STA AtariOS::DTIMLO
	LDA #<(TBYT)
	STA AtariOS::DBYT
	LDA #>(TBYT)
	STA AtariOS::DBYT + 1


	; ===============================================================================================
	; Read sectors 4-720 in a loop forever
TLOOP0:
	LDA #4
	STA AtariOS::DAUX1
	LDA #0
	STA AtariOS::DAUX2

TLOOP:
	LDA GTIA_R::CONSOL
	CMP #6
	BEQ TLOOP

	LDA #TDIR
	STA AtariOS::DSTATS

	LDA #HIMODE
	JSR PHISIO
	BMI TERROR

	LDA GTIA_R::CONSOL
	CMP #3
	BEQ TEND

	INC AtariOS::DAUX1
	BNE TLOOP1
	INC AtariOS::DAUX2
TLOOP1:
	LDA AtariOS::DAUX2
	CMP #2
	BNE TLOOP
	LDA AtariOS::DAUX1
	CMP #$D1
	BNE TLOOP
	JMP TLOOP0

TERROR:
	; Display an error message
	BPUT 0, TERRL, TERR


	; ===============================================================================================
	; End the diagnostic
TEND:
	; Display a message about it
	BPUT 0, TAGAL, TAGA

	; Get a key and exit if escape is pressed
	JSR GETKEY
	CMP #27
	BEQ EXIT
	JMP MAIN

EXIT:
	JMP AtariOS::COLDSV
; End MAIN



; *************************************************************************************************
; Displays a character.
; Incoming:
;   A               Byte, The character to display.
; Outgoing:
;   All registers are lost.
PUTCHR:
	TAY
	LDA AtariOS::EOUTCHV + 1
	PHA
	LDA AtariOS::EOUTCHV
	PHA
	TYA
	RTS
; End PUTCHR



; *************************************************************************************************
; Gets a key from the user.
; Outgoing:
;   A is the key pressed.
GETKEY:
	LDA AtariOS::KGETCHV + 1
	PHA
	LDA AtariOS::KGETCHV
	PHA
	; Fall through ------>
; End GETKEY



; *************************************************************************************************
; Just an RTS.
; Outgoing: All registers are preserved.
XRTS:
	RTS
; End XRTS



; *************************************************************************************************
; Data

MTXT:
	.BYTE 125
	.BYTE "highspeed SIO entry test",ATASCII::EndOfLine
MTXTL	= * - MTXT

TNOPATCH:
	.BYTE "patch not detected",ATASCII::EndOfLine
TNOPATCHL	= * - TNOPATCH

TFOUND:
	.BYTE "patch detected: "
TFOUNDL	= * - TFOUND

TOLD:
	.BYTE "patch is too old", ATASCII::EndOfLine
TOLDL	= * - TOLD

TVEROK:
	.BYTE "Version OK, starting test",ATASCII::EndOfLine
TVEROKL	= * - TVEROK

TERR:
	.BYTE "Error, aborting test",ATASCII::EndOfLine
TERRL	= * - TERR

TAGA:
	.BYTE ATASCII::EndOfLine, "press ESC to reboot or any",ATASCII::EndOfLine
	.BYTE "other key to restart test",ATASCII::EndOfLine
TAGAL	= * - TAGA

; assemble highspeed code at identical page-boundaries as hipatch code
.IF (* & 127) <> 0
	.RES (128 - (* & 127)), 0
.ENDIF

END:
