; Support routines for debbuging using an external display
; License: GPL-2, see LICENSE
; This Version (c) 2022 Mark Ormston <memso@memso.net>
; Original Copyright (c) 2005 Matthias Reichl

.IFDEF HWDEBUG

; *************************************************************************************************
; Write hex character in A to position X,X + 1
; Incoming:
;   A               Byte, The value to convert to hex.
;   X               Byte, The offset into the hardware display to use.
; Outgoing:
;   A and Y are lost.
;   X is 2 higher for the next character after the data just displayed.
SDBGHEX:
	PHA

	; Output the high byte
	LSR
	LSR
	LSR
	LSR
	TAY
	LDA HEXTAB,Y
	STA HWDISP,X
	INX

	; Output the low byte
	PLA
	AND #$0F
	TAY
	LDA HEXTAB,Y
	STA HWDISP,X
	INX

	RTS
; End SDBGHEX



; *************************************************************************************************

; 16 bytes, Converts from a nibble to a hex character.
HEXTAB:
	.BYTE "0123456789ABCDEF"



; *************************************************************************************************
; Clears the 8 character debug display to all spaces.
; Outgoing:
;   A and X are lost.
SDBGCLR:
	LDX #0
	LDA #' '
SDBGCLL:
	STA HWDISP,X
	INX
	CPX #8
	BNE SDBGCLL
	RTS
; End SDBGCLR

.ENDIF
