; Disk transfer speed test
; Time reading sectors 256-511
; License: GPL-2, see LICENSE
; This Version (c) 2022 Mark Ormston <memso@memso.net>
; Original copyright (c) by Matthias Reichl <hias@horus.com>

.INCLUDE "../Include/HISIO.inc"
.INCLUDE "../Include/CIO.inc"

; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Addresses

; Start address.
START	= $2000



; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Code and Data

; !!! Making it work. This adds a DOS header for the main code
.ORG START - 6
AtariDOS_BinaryStart START, MYEND - START

; *************************************************************************************************
; Start the test.
;Start:
	LDA #'1'
	STA AtariOS::DDEVIC
	LDA #1
	STA AtariOS::DUNIT
	LDA #7
	STA AtariOS::DTIMLO

	LDA #'R'
	STA AtariOS::DCOMND

	LDA #0
	STA AtariOS::DBYT
	LDA #1
	STA AtariOS::DBYT + 1

	LDA #<(BUF)
	STA AtariOS::DBUF
	LDA #>(BUF)
	STA AtariOS::DBUF + 1

	LDA #0
	STA AtariOS::DAUX1
	LDA #1
	STA AtariOS::DAUX2


	; ===============================================================================================
	; Timing loop
TLOOP:
	LDA ANTIC_R::VCOUNT
	BNE TLOOP
	STA ANTIC_W::WSYNC

	LDA #0
	STA AtariOS::ZP_RTCLOK + 2
	STA AtariOS::ZP_RTCLOK + 1
	STA AtariOS::ZP_RTCLOK

TLOOP1:
	LDA #$40
	STA AtariOS::DSTATS
	JSR AtariOS::SIOV
	BPL TOK

	PRINT 0,TERROR
	JMP TLOOP

TOK:
	INC AtariOS::DAUX1
	BNE TLOOP1


	; ===============================================================================================
	; Display the results

	; Save ANTIC_R::VCOUNT when this ended
	LDA ANTIC_R::VCOUNT
	STA VLINE

	; Display the AtariOS::ZP_RTCLOK value
	LDA AtariOS::ZP_RTCLOK + 2
	STA VTIM
	LDA AtariOS::ZP_RTCLOK + 1
	STA VTIM + 1
	LDA AtariOS::ZP_RTCLOK
	STA VTIM + 2
	JSR PHEX
	LDA VTIM + 1
	JSR PHEX
	LDA VTIM
	JSR PHEX

	; Display the ANTIC_R::VCOUNT value
	LDA #'.'
	JSR PUTCHAR
	LDA VLINE
	JSR PHEX
	LDA #ATASCII::EndOfLine
	JSR PUTCHAR

	; Restart
	JMP TLOOP
; End Start



; *************************************************************************************************
; Prints a hex value.
; Incoming:
;   A               Byte, The hex value to output.
; Outgoing: All registers are lost.
PHEX:
	PHA
	LSR
	LSR
	LSR
	LSR
	TAX
	LDA HEXTAB,X
	JSR PUTCHAR
	PLA
	AND #$0F
	TAX
	LDA HEXTAB,X
; End PHEX



; *************************************************************************************************
; Displays a single character.
; Incoming:
;   A               Byte, The character to display.
; Outgoing: All registers are lost.
PUTCHAR:
	TAY
	LDA AtariOS::EOUTCHV + 1
	PHA
	LDA AtariOS::EOUTCHV
	PHA
	TYA
	RTS
; End PUTCHAR



; *************************************************************************************************
; Data

; 16 bytes, Converts from a nibble to a hexadecimal character.
HEXTAB:
	.BYTE "0123456789ABCDEF"

; String, SIO error message.
TERROR:
	.BYTE "SIO error, restarting test", ATASCII::EndOfLine

VLINE:
	.BYTE 0
VTIM:
	.BYTE 0,0,0

MYEND:
BUF	= *



; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Store where the program starts for DOS

; !!! Making it work. Adds the start vector for DOS
.ORG AtariDOS::RUNAD
AtariDOS_BinaryBlock AtariDOS::RUNAD, 2
.WORD START
